@@ -1,14 +1,16 @@
+from __future__ import annotations
+
 import os
 import random
 import subprocess
 import sys
 from datetime import datetime
 from pathlib import Path
-from typing import Tuple
+from typing import Tuple, Union
 
 import aiosqlite
 import click
-from blspy import AugSchemeMPL, G1Element, G2Element
+from chinillablspy import AugSchemeMPL, G1Element, G2Element
 
 from chinilla.consensus.coinbase import create_farmer_coin, create_pool_coin
 from chinilla.consensus.default_constants import DEFAULT_CONSTANTS
@@ -178,7 +180,7 @@ def rand_full_block() -> FullBlock:
     return full_block
 
 
-async def setup_db(name: str, db_version: int) -> DBWrapper2:
+async def setup_db(name: Union[str, os.PathLike], db_version: int) -> DBWrapper2:
     db_filename = Path(name)
     try:
         os.unlink(db_filename)
