@@ -5,13 +5,13 @@ set -o errexit -o nounset
 git status
 
 echo "Installing global npm packages"
-cd npm_macos || exit
+cd npm_macos || exit 1
 npm ci
 PATH=$(npm bin):$PATH
-cd ../../ || exit
+cd ../../ || exit 1
 git submodule update --init chinilla-blockchain-gui
 
-cd ./chinilla-blockchain-gui || exit
+cd ./chinilla-blockchain-gui || exit 1
 echo "npm build"
 lerna clean -y
 npm ci
@@ -38,7 +38,7 @@ rm -rf packages/icons
 rm -rf packages/wallets
 
 # Remove unused fat npm modules from the gui package
-cd ./packages/gui/node_modules
+cd ./packages/gui/node_modules || exit 1
 echo "Remove unused node_modules in the gui package to make cache slim more"
 rm -rf electron/dist # ~186MB
 rm -rf "@mui" # ~71MB
