@@ -1,15 +1,12 @@
 from typing import Any, Dict, List, Optional
 
-import aiohttp
 
+from chinilla.cmds.cmds_util import get_any_service_client
 from chinilla.cmds.units import units
 from chinilla.consensus.block_record import BlockRecord
 from chinilla.rpc.farmer_rpc_client import FarmerRpcClient
 from chinilla.rpc.full_node_rpc_client import FullNodeRpcClient
 from chinilla.rpc.wallet_rpc_client import WalletRpcClient
-from chinilla.util.config import load_config
-from chinilla.util.default_root import DEFAULT_ROOT_PATH
-from chinilla.util.ints import uint16
 from chinilla.util.misc import format_bytes
 from chinilla.util.misc import format_minutes
 from chinilla.util.network import is_localhost
@@ -18,146 +15,64 @@ SECONDS_PER_BLOCK = (24 * 3600) / 4608
 
 
 async def get_harvesters_summary(farmer_rpc_port: Optional[int]) -> Optional[Dict[str, Any]]:
-    try:
-        config = load_config(DEFAULT_ROOT_PATH, "config.yaml")
-        self_hostname = config["self_hostname"]
-        if farmer_rpc_port is None:
-            farmer_rpc_port = config["farmer"]["rpc_port"]
-        farmer_client = await FarmerRpcClient.create(self_hostname, uint16(farmer_rpc_port), DEFAULT_ROOT_PATH, config)
-        plots = await farmer_client.get_harvesters_summary()
-    except Exception as e:
-        if isinstance(e, aiohttp.ClientConnectorError):
-            print(f"Connection error. Check if farmer is running at {farmer_rpc_port}")
-        else:
-            print(f"Exception from 'harvester' {e}")
-        return None
-    farmer_client.close()
-    await farmer_client.await_closed()
-    return plots
+    farmer_client: Optional[FarmerRpcClient]
+    async with get_any_service_client("farmer", farmer_rpc_port) as node_config_fp:
+        farmer_client, _, _ = node_config_fp
+        if farmer_client is not None:
+            return await farmer_client.get_harvesters_summary()
+    return None
 
 
 async def get_blockchain_state(rpc_port: Optional[int]) -> Optional[Dict[str, Any]]:
-    blockchain_state = None
-    try:
-        config = load_config(DEFAULT_ROOT_PATH, "config.yaml")
-        self_hostname = config["self_hostname"]
-        if rpc_port is None:
-            rpc_port = config["full_node"]["rpc_port"]
-        client = await FullNodeRpcClient.create(self_hostname, uint16(rpc_port), DEFAULT_ROOT_PATH, config)
-        blockchain_state = await client.get_blockchain_state()
-    except Exception as e:
-        if isinstance(e, aiohttp.ClientConnectorError):
-            print(f"Connection error. Check if full node is running at {rpc_port}")
-        else:
-            print(f"Exception from 'full node' {e}")
-
-    client.close()
-    await client.await_closed()
-    return blockchain_state
+    client: Optional[FullNodeRpcClient]
+    async with get_any_service_client("full_node", rpc_port) as node_config_fp:
+        client, _, _ = node_config_fp
+        if client is not None:
+            return await client.get_blockchain_state()
+    return None
 
 
 async def get_average_block_time(rpc_port: Optional[int]) -> float:
-    try:
-        blocks_to_compare = 500
-        config = load_config(DEFAULT_ROOT_PATH, "config.yaml")
-        self_hostname = config["self_hostname"]
-        if rpc_port is None:
-            rpc_port = config["full_node"]["rpc_port"]
-        client = await FullNodeRpcClient.create(self_hostname, uint16(rpc_port), DEFAULT_ROOT_PATH, config)
-        blockchain_state = await client.get_blockchain_state()
-        curr: Optional[BlockRecord] = blockchain_state["peak"]
-        if curr is None or curr.height < (blocks_to_compare + 100):
-            client.close()
-            await client.await_closed()
-            return SECONDS_PER_BLOCK
-        while curr is not None and curr.height > 0 and not curr.is_transaction_block:
-            curr = await client.get_block_record(curr.prev_hash)
-        if curr is None:
-            client.close()
-            await client.await_closed()
-            return SECONDS_PER_BLOCK
-
-        past_curr = await client.get_block_record_by_height(curr.height - blocks_to_compare)
-        while past_curr is not None and past_curr.height > 0 and not past_curr.is_transaction_block:
-            past_curr = await client.get_block_record(past_curr.prev_hash)
-        if past_curr is None:
-            client.close()
-            await client.await_closed()
-            return SECONDS_PER_BLOCK
-
-        client.close()
-        await client.await_closed()
-        return (curr.timestamp - past_curr.timestamp) / (curr.height - past_curr.height)
-
-    except Exception as e:
-        if isinstance(e, aiohttp.ClientConnectorError):
-            print(f"Connection error. Check if full node is running at {rpc_port}")
-        else:
-            print(f"Exception from 'full node' {e}")
-
-    client.close()
-    await client.await_closed()
+    client: Optional[FullNodeRpcClient]
+    async with get_any_service_client("full_node", rpc_port) as node_config_fp:
+        client, _, _ = node_config_fp
+        if client is not None:
+            blocks_to_compare = 500
+            blockchain_state = await client.get_blockchain_state()
+            curr: Optional[BlockRecord] = blockchain_state["peak"]
+            if curr is None or curr.height < (blocks_to_compare + 100):
+                return SECONDS_PER_BLOCK
+            while curr is not None and curr.height > 0 and not curr.is_transaction_block:
+                curr = await client.get_block_record(curr.prev_hash)
+            if curr is None or curr.timestamp is None or curr.height is None:
+                # stupid mypy
+                return SECONDS_PER_BLOCK
+            past_curr = await client.get_block_record_by_height(curr.height - blocks_to_compare)
+            while past_curr is not None and past_curr.height > 0 and not past_curr.is_transaction_block:
+                past_curr = await client.get_block_record(past_curr.prev_hash)
+            if past_curr is None or past_curr.timestamp is None or past_curr.height is None:
+                # stupid mypy
+                return SECONDS_PER_BLOCK
+            return (curr.timestamp - past_curr.timestamp) / (curr.height - past_curr.height)
     return SECONDS_PER_BLOCK
 
 
 async def get_wallets_stats(wallet_rpc_port: Optional[int]) -> Optional[Dict[str, Any]]:
-    amounts = None
-    try:
-        config = load_config(DEFAULT_ROOT_PATH, "config.yaml")
-        self_hostname = config["self_hostname"]
-        if wallet_rpc_port is None:
-            wallet_rpc_port = config["wallet"]["rpc_port"]
-        wallet_client = await WalletRpcClient.create(self_hostname, uint16(wallet_rpc_port), DEFAULT_ROOT_PATH, config)
-        amounts = await wallet_client.get_farmed_amount()
-    #
-    # Don't catch any exceptions, the caller will handle it
-    #
-    finally:
-        wallet_client.close()
-        await wallet_client.await_closed()
-
-    return amounts
-
-
-async def is_farmer_running(farmer_rpc_port: Optional[int]) -> bool:
-    is_running = False
-    try:
-        config = load_config(DEFAULT_ROOT_PATH, "config.yaml")
-        self_hostname = config["self_hostname"]
-        if farmer_rpc_port is None:
-            farmer_rpc_port = config["farmer"]["rpc_port"]
-        farmer_client = await FarmerRpcClient.create(self_hostname, uint16(farmer_rpc_port), DEFAULT_ROOT_PATH, config)
-        await farmer_client.get_connections()
-        is_running = True
-    except Exception as e:
-        if isinstance(e, aiohttp.ClientConnectorError):
-            print(f"Connection error. Check if farmer is running at {farmer_rpc_port}")
-        else:
-            print(f"Exception from 'farmer' {e}")
-
-    farmer_client.close()
-    await farmer_client.await_closed()
-    return is_running
+    wallet_client: Optional[WalletRpcClient]
+    async with get_any_service_client("wallet", wallet_rpc_port, login_to_wallet=False) as node_config_fp:
+        wallet_client, _, _ = node_config_fp
+        if wallet_client is not None:
+            return await wallet_client.get_farmed_amount()
+    return None
 
 
 async def get_challenges(farmer_rpc_port: Optional[int]) -> Optional[List[Dict[str, Any]]]:
-    signage_points = None
-    try:
-        config = load_config(DEFAULT_ROOT_PATH, "config.yaml")
-        self_hostname = config["self_hostname"]
-        if farmer_rpc_port is None:
-            farmer_rpc_port = config["farmer"]["rpc_port"]
-        farmer_client = await FarmerRpcClient.create(self_hostname, uint16(farmer_rpc_port), DEFAULT_ROOT_PATH, config)
-        signage_points = await farmer_client.get_signage_points()
-    except Exception as e:
-        if isinstance(e, aiohttp.ClientConnectorError):
-            print(f"Connection error. Check if farmer is running at {farmer_rpc_port}")
-        else:
-            print(f"Exception from 'farmer' {e}")
-
-    farmer_client.close()
-    await farmer_client.await_closed()
-    return signage_points
+    farmer_client: Optional[FarmerRpcClient]
+    async with get_any_service_client("farmer", farmer_rpc_port) as node_config_fp:
+        farmer_client, _, _ = node_config_fp
+        if farmer_client is not None:
+            return await farmer_client.get_signage_points()
+    return None
 
 
 async def challenges(farmer_rpc_port: Optional[int], limit: int) -> None:
@@ -186,18 +101,15 @@ async def summary(
 ) -> None:
     harvesters_summary = await get_harvesters_summary(farmer_rpc_port)
     blockchain_state = await get_blockchain_state(rpc_port)
-    farmer_running = await is_farmer_running(farmer_rpc_port)
+    farmer_running = False if harvesters_summary is None else True  # harvesters uses farmer rpc too
 
     wallet_not_ready: bool = False
-    wallet_not_running: bool = False
     amounts = None
     try:
         amounts = await get_wallets_stats(wallet_rpc_port)
-    except Exception as e:
-        if isinstance(e, aiohttp.ClientConnectorError):
-            wallet_not_running = True
-        else:
-            wallet_not_ready = True
+    except Exception:
+        wallet_not_ready = True
+    wallet_not_running: bool = True if amounts is None else False
 
     print("Farming status: ", end="")
     if blockchain_state is None:
@@ -279,8 +191,8 @@ async def summary(
     if amounts is None:
         if wallet_not_running:
             print(
-                "For details on farmed rewards and fees you should run 'chinilla start wallet' and 'chinilla wallet "
-                "show'"
+                "For details on farmed rewards and fees you should run"
+                " 'chinilla start wallet' and 'chinilla wallet show'"
             )
         elif wallet_not_ready:
             print("For details on farmed rewards and fees you should run 'chinilla wallet show'")
