@@ -2,20 +2,24 @@ import logging
 import os
 import sys
 
-from blspy import AugSchemeMPL, G1Element, G2Element, PrivateKey
+from chinillablspy import AugSchemeMPL, G1Element, G2Element, PrivateKey
 from enum import Enum
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 from chinilla.consensus.coinbase import create_puzzlehash_for_pk
+from chinilla.cmds.passphrase_funcs import obtain_current_passphrase
 from chinilla.daemon.client import connect_to_daemon_and_validate
 from chinilla.daemon.keychain_proxy import KeychainProxy, connect_to_keychain_and_validate, wrap_local_keychain
 from chinilla.util.bech32m import encode_puzzle_hash
 from chinilla.util.errors import KeychainNotSet
 from chinilla.util.config import load_config
 from chinilla.util.default_root import DEFAULT_ROOT_PATH
+from chinilla.util.errors import KeychainException
+from chinilla.util.file_keyring import MAX_LABEL_LENGTH
 from chinilla.util.ints import uint32
-from chinilla.util.keychain import Keychain, bytes_to_mnemonic, generate_mnemonic, mnemonic_to_seed, unlocks_keyring
+from chinilla.util.keychain import Keychain, bytes_to_mnemonic, generate_mnemonic, mnemonic_to_seed
+from chinilla.util.keyring_wrapper import KeyringWrapper
 from chinilla.wallet.derive_keys import (
     master_sk_to_farmer_sk,
     master_sk_to_pool_sk,
@@ -24,6 +28,19 @@ from chinilla.wallet.derive_keys import (
 )
 
 
+def unlock_keyring() -> None:
+    """
+    Used to unlock the keyring interactively, if necessary
+    """
+
+    try:
+        if KeyringWrapper.get_shared_instance().has_master_passphrase():
+            obtain_current_passphrase(use_passphrase_cache=True)
+    except Exception as e:
+        print(f"Unable to unlock the keyring: {e}")
+        sys.exit(1)
+
+
 def generate_and_print():
     """
     Generates a seed for a private key, and prints the mnemonic to the terminal.
@@ -36,61 +53,103 @@ def generate_and_print():
     return mnemonic
 
 
-@unlocks_keyring(use_passphrase_cache=True)
-def generate_and_add():
+def generate_and_add(label: Optional[str]):
     """
     Generates a seed for a private key, prints the mnemonic to the terminal, and adds the key to the keyring.
     """
-
-    mnemonic = generate_mnemonic()
+    unlock_keyring()
     print("Generating private key")
-    add_private_key_seed(mnemonic)
+    query_and_add_private_key_seed(mnemonic=generate_mnemonic(), label=label)
 
 
-@unlocks_keyring(use_passphrase_cache=True)
-def query_and_add_private_key_seed():
-    mnemonic = input("Enter the mnemonic you want to use: ")
-    add_private_key_seed(mnemonic)
+def query_and_add_private_key_seed(mnemonic: Optional[str], label: Optional[str] = None):
+    unlock_keyring()
+    if mnemonic is None:
+        mnemonic = input("Enter the mnemonic you want to use: ")
+    if label is None:
+        label = input("Enter the label you want to assign to this key (Press Enter to skip): ")
+    if len(label) == 0:
+        label = None
+    add_private_key_seed(mnemonic, label)
 
 
-@unlocks_keyring(use_passphrase_cache=True)
-def add_private_key_seed(mnemonic: str):
+def add_private_key_seed(mnemonic: str, label: Optional[str]):
     """
-    Add a private key seed to the keyring, with the given mnemonic.
+    Add a private key seed to the keyring, with the given mnemonic and an optional label.
     """
-
+    unlock_keyring()
     try:
-        passphrase = ""
-        sk = Keychain().add_private_key(mnemonic, passphrase)
+        sk = Keychain().add_private_key(mnemonic, label)
         fingerprint = sk.get_g1().get_fingerprint()
         print(f"Added private key with public key fingerprint {fingerprint}")
 
-    except ValueError as e:
+    except (ValueError, KeychainException) as e:
         print(e)
-        return None
 
 
-@unlocks_keyring(use_passphrase_cache=True)
+def show_all_key_labels() -> None:
+    unlock_keyring()
+    fingerprint_width = 11
+
+    def print_line(fingerprint: str, label: str) -> None:
+        fingerprint_text = ("{0:<" + str(fingerprint_width) + "}").format(fingerprint)
+        label_text = ("{0:<" + str(MAX_LABEL_LENGTH) + "}").format(label)
+        print("| " + fingerprint_text + " | " + label_text + " |")
+
+    keys = Keychain().get_keys()
+
+    if len(keys) == 0:
+        sys.exit("No keys are present in the keychain. Generate them with 'chinilla keys generate'")
+
+    print_line("fingerprint", "label")
+    print_line("-" * fingerprint_width, "-" * MAX_LABEL_LENGTH)
+
+    for key_data in keys:
+        print_line(str(key_data.fingerprint), key_data.label or "No label assigned")
+
+
+def set_key_label(fingerprint: int, label: str) -> None:
+    unlock_keyring()
+    try:
+        Keychain().set_label(fingerprint, label)
+        print(f"label {label!r} assigned to {fingerprint!r}")
+    except Exception as e:
+        sys.exit(f"Error: {e}")
+
+
+def delete_key_label(fingerprint: int) -> None:
+    unlock_keyring()
+    try:
+        Keychain().delete_label(fingerprint)
+        print(f"label removed for {fingerprint!r}")
+    except Exception as e:
+        sys.exit(f"Error: {e}")
+
+
 def show_all_keys(show_mnemonic: bool, non_observer_derivation: bool):
     """
     Prints all keys and mnemonics (if available).
     """
+    unlock_keyring()
     root_path = DEFAULT_ROOT_PATH
     config = load_config(root_path, "config.yaml")
-    private_keys = Keychain().get_all_private_keys()
+    all_keys = Keychain().get_keys(True)
     selected = config["selected_network"]
     prefix = config["network_overrides"]["config"][selected]["address_prefix"]
-    if len(private_keys) == 0:
+    if len(all_keys) == 0:
         print("There are no saved private keys")
         return None
     msg = "Showing all public keys derived from your master seed and private key:"
     if show_mnemonic:
         msg = "Showing all public and private keys"
     print(msg)
-    for sk, seed in private_keys:
+    for key_data in all_keys:
+        sk = key_data.private_key
         print("")
-        print("Fingerprint:", sk.get_g1().get_fingerprint())
-        print("Master public key (m):", sk.get_g1())
+        if key_data.label is not None:
+            print("Label:", key_data.label)
+        print("Fingerprint:", key_data.fingerprint)
+        print("Master public key (m):", key_data.public_key)
         print(
             "Farmer public key (m/12381/8444/0/0):",
             master_sk_to_farmer_sk(sk).get_g1(),
@@ -103,23 +162,22 @@ def show_all_keys(show_mnemonic: bool, non_observer_derivation: bool):
         )
         wallet_address: str = encode_puzzle_hash(create_puzzlehash_for_pk(first_wallet_sk.get_g1()), prefix)
         print(f"First wallet address{' (non-observer)' if non_observer_derivation else ''}: {wallet_address}")
-        assert seed is not None
         if show_mnemonic:
             print("Master private key (m):", bytes(sk).hex())
             print(
                 "First wallet secret key (m/12381/8444/2/0):",
                 master_sk_to_wallet_sk(sk, uint32(0)),
             )
-            mnemonic = bytes_to_mnemonic(seed)
+            mnemonic = bytes_to_mnemonic(key_data.entropy)
             print("  Mnemonic seed (24 secret words):")
             print(mnemonic)
 
 
-@unlocks_keyring(use_passphrase_cache=True)
 def delete(fingerprint: int):
     """
     Delete a key by its public key fingerprint (which is an integer).
     """
+    unlock_keyring()
     print(f"Deleting private_key with fingerprint {fingerprint}")
     Keychain().delete_key_by_fingerprint(fingerprint)
 
@@ -242,7 +300,7 @@ async def migrate_keys(root_path: Path, forced: bool = False) -> bool:
 
             for sk, seed_bytes in keys_to_migrate:
                 mnemonic = bytes_to_mnemonic(seed_bytes)
-                await keychain_proxy.add_private_key(mnemonic, "")
+                await keychain_proxy.add_private_key(mnemonic)
                 fingerprint = sk.get_g1().get_fingerprint()
                 print(f"Added private key with public key fingerprint {fingerprint}")
 
@@ -655,8 +713,8 @@ def derive_child_key(
                 print(f"{key_type_str} private key {i}{hd_path}: {private_key_string_repr(sk)}")
 
 
-@unlocks_keyring(use_passphrase_cache=True)
 def private_key_for_fingerprint(fingerprint: int) -> Optional[PrivateKey]:
+    unlock_keyring()
     private_keys = Keychain().get_all_private_keys()
 
     for sk, _ in private_keys:
@@ -703,7 +761,7 @@ def private_key_from_mnemonic_seed_file(filename: Path) -> PrivateKey:
     """
 
     mnemonic = filename.read_text().rstrip()
-    seed = mnemonic_to_seed(mnemonic, "")
+    seed = mnemonic_to_seed(mnemonic)
     return AugSchemeMPL.key_gen(seed)
 
 
