@@ -24,8 +24,8 @@ from chinilla.util.bech32m import encode_puzzle_hash, decode_puzzle_hash
 from chinilla.util.byte_types import hexstr_to_bytes
 from chinilla.util.config import load_config
 from chinilla.util.default_root import DEFAULT_ROOT_PATH
-from chinilla.util.ints import uint16, uint32, uint64
-from chinilla.cmds.cmds_util import transaction_submitted_msg, transaction_status_msg
+from chinilla.util.ints import uint32, uint64
+from chinilla.cmds.cmds_util import transaction_submitted_msg, transaction_status_msg, get_any_service_client
 from chinilla.wallet.transaction_record import TransactionRecord
 from chinilla.wallet.util.wallet_types import WalletType
 
@@ -174,89 +174,65 @@ async def pprint_pool_wallet_state(
 
 
 async def show(args: dict, wallet_client: WalletRpcClient, fingerprint: int) -> None:
-
-    config = load_config(DEFAULT_ROOT_PATH, "config.yaml")
-    self_hostname = config["self_hostname"]
-    farmer_rpc_port = config["farmer"]["rpc_port"]
-    farmer_client = await FarmerRpcClient.create(self_hostname, uint16(farmer_rpc_port), DEFAULT_ROOT_PATH, config)
-    address_prefix = config["network_overrides"]["config"][config["selected_network"]]["address_prefix"]
-    summaries_response = await wallet_client.get_wallets()
-    wallet_id_passed_in = args.get("id", None)
-    try:
-        pool_state_list = (await farmer_client.get_pool_state())["pool_state"]
-    except Exception as e:
-        if isinstance(e, aiohttp.ClientConnectorError):
-            print(
-                f"Connection error. Check if farmer is running at {farmer_rpc_port}."
-                f" You can run the farmer by:\n    chinilla start farmer-only"
-            )
-        else:
-            print(f"Exception from 'wallet' {e}")
-        farmer_client.close()
-        await farmer_client.await_closed()
-        return
-    pool_state_dict: Dict[bytes32, Dict] = {
-        bytes32.from_hexstr(pool_state_item["pool_config"]["launcher_id"]): pool_state_item
-        for pool_state_item in pool_state_list
-    }
-    if wallet_id_passed_in is not None:
-        for summary in summaries_response:
-            typ = WalletType(int(summary["type"]))
-            if summary["id"] == wallet_id_passed_in and typ != WalletType.POOLING_WALLET:
-                print(f"Wallet with id: {wallet_id_passed_in} is not a pooling wallet. Please provide a different id.")
-                return
-        pool_wallet_info, _ = await wallet_client.pw_status(wallet_id_passed_in)
-        await pprint_pool_wallet_state(
-            wallet_client,
-            wallet_id_passed_in,
-            pool_wallet_info,
-            address_prefix,
-            pool_state_dict.get(pool_wallet_info.launcher_id),
-        )
-    else:
-        print(f"Wallet height: {await wallet_client.get_height_info()}")
-        print(f"Sync status: {'Synced' if (await wallet_client.get_synced()) else 'Not synced'}")
-        for summary in summaries_response:
-            wallet_id = summary["id"]
-            typ = WalletType(int(summary["type"]))
-            if typ == WalletType.POOLING_WALLET:
-                print(f"Wallet id {wallet_id}: ")
-                pool_wallet_info, _ = await wallet_client.pw_status(wallet_id)
+    farmer_client: Optional[FarmerRpcClient]
+    async with get_any_service_client("farmer") as node_config_fp:
+        farmer_client, config, _ = node_config_fp
+        if farmer_client is not None:
+            address_prefix = config["network_overrides"]["config"][config["selected_network"]]["address_prefix"]
+            summaries_response = await wallet_client.get_wallets()
+            wallet_id_passed_in = args.get("id", None)
+            pool_state_list = (await farmer_client.get_pool_state())["pool_state"]
+            pool_state_dict: Dict[bytes32, Dict] = {
+                bytes32.from_hexstr(pool_state_item["pool_config"]["launcher_id"]): pool_state_item
+                for pool_state_item in pool_state_list
+            }
+            if wallet_id_passed_in is not None:
+                for summary in summaries_response:
+                    typ = WalletType(int(summary["type"]))
+                    if summary["id"] == wallet_id_passed_in and typ != WalletType.POOLING_WALLET:
+                        print(
+                            f"Wallet with id: {wallet_id_passed_in} is not a pooling wallet."
+                            " Please provide a different id."
+                        )
+                        return
+                pool_wallet_info, _ = await wallet_client.pw_status(wallet_id_passed_in)
                 await pprint_pool_wallet_state(
                     wallet_client,
-                    wallet_id,
+                    wallet_id_passed_in,
                     pool_wallet_info,
                     address_prefix,
                     pool_state_dict.get(pool_wallet_info.launcher_id),
                 )
-                print("")
-    farmer_client.close()
-    await farmer_client.await_closed()
+            else:
+                print(f"Wallet height: {await wallet_client.get_height_info()}")
+                print(f"Sync status: {'Synced' if (await wallet_client.get_synced()) else 'Not synced'}")
+                for summary in summaries_response:
+                    wallet_id = summary["id"]
+                    typ = WalletType(int(summary["type"]))
+                    if typ == WalletType.POOLING_WALLET:
+                        print(f"Wallet id {wallet_id}: ")
+                        pool_wallet_info, _ = await wallet_client.pw_status(wallet_id)
+                        await pprint_pool_wallet_state(
+                            wallet_client,
+                            wallet_id,
+                            pool_wallet_info,
+                            address_prefix,
+                            pool_state_dict.get(pool_wallet_info.launcher_id),
+                        )
+                        print("")
 
 
 async def get_login_link(launcher_id_str: str) -> None:
     launcher_id: bytes32 = bytes32.from_hexstr(launcher_id_str)
-    config = load_config(DEFAULT_ROOT_PATH, "config.yaml")
-    self_hostname = config["self_hostname"]
-    farmer_rpc_port = config["farmer"]["rpc_port"]
-    farmer_client = await FarmerRpcClient.create(self_hostname, uint16(farmer_rpc_port), DEFAULT_ROOT_PATH, config)
-    try:
-        login_link: Optional[str] = await farmer_client.get_pool_login_link(launcher_id)
-        if login_link is None:
-            print("Was not able to get login link.")
-        else:
-            print(login_link)
-    except Exception as e:
-        if isinstance(e, aiohttp.ClientConnectorError):
-            print(
-                f"Connection error. Check if farmer is running at {farmer_rpc_port}."
-                f" You can run the farmer by:\n    chinilla start farmer-only"
-            )
-        else:
-            print(f"Exception from 'farmer' {e}")
-    finally:
-        farmer_client.close()
-        await farmer_client.await_closed()
+    farmer_client: Optional[FarmerRpcClient]
+    async with get_any_service_client("farmer") as node_config_fp:
+        farmer_client, _, _ = node_config_fp
+        if farmer_client is not None:
+            login_link: Optional[str] = await farmer_client.get_pool_login_link(launcher_id)
+            if login_link is None:
+                print("Was not able to get login link.")
+            else:
+                print(login_link)
 
 
 async def submit_tx_with_confirmation(
