@@ -4,7 +4,7 @@ import ssl
 import traceback
 
 from aiohttp import ClientSession, ClientConnectorError
-from blspy import AugSchemeMPL, PrivateKey
+from chinillablspy import AugSchemeMPL, PrivateKey
 from chinilla.cmds.init_funcs import check_keys
 from chinilla.daemon.client import DaemonProxy
 from chinilla.daemon.keychain_server import (
@@ -22,7 +22,7 @@ from chinilla.util.errors import (
     KeychainKeyNotFound,
     KeychainMalformedRequest,
     KeychainMalformedResponse,
-    KeychainProxyConnectionFailure,
+    KeychainProxyConnectionTimeout,
 )
 from chinilla.util.keychain import (
     Keychain,
@@ -45,13 +45,13 @@ class KeychainProxy(DaemonProxy):
     def __init__(
         self,
         log: logging.Logger,
-        uri: Optional[str] = None,
+        uri: str = "",
         ssl_context: Optional[ssl.SSLContext] = None,
         local_keychain: Optional[Keychain] = None,
         user: Optional[str] = None,
         service: Optional[str] = None,
     ):
-        super().__init__(uri or "", ssl_context)
+        super().__init__(uri, ssl_context)
         self.log = log
         if local_keychain:
             self.keychain = local_keychain
@@ -61,7 +61,7 @@ class KeychainProxy(DaemonProxy):
         self.keychain_service = service
         # these are used to track and close the keychain connection
         self.keychain_connection_task: Optional[asyncio.Task[None]] = None
-        self.disconnect: bool = False
+        self.shut_down: bool = False
         self.connection_established: asyncio.Event = asyncio.Event()
 
     def use_local_keychain(self) -> bool:
@@ -88,21 +88,21 @@ class KeychainProxy(DaemonProxy):
         Overrides DaemonProxy._get() to handle the connection state
         """
         try:
-            if not self.disconnect:  # if we are disconnected, and we send a request we should throw original error.
-                await asyncio.wait_for(self.connection_established.wait(), timeout=10)  # is 10 seconds too long?
+            if not self.shut_down:  # if we are shut down, and we send a request we should throw original error.
+                await asyncio.wait_for(self.connection_established.wait(), timeout=30)  # in case of heavy swap usage.
             else:
                 self.log.error("Attempting to send request to a keychain-proxy that has shut down.")
             self.log.debug(f"Sending request to keychain command: {request['command']} from {request['origin']}.")
             return await super()._get(request)
         except asyncio.TimeoutError:
-            raise KeychainProxyConnectionFailure()
+            raise KeychainProxyConnectionTimeout()
 
     async def start(self) -> None:
         self.keychain_connection_task = asyncio.create_task(self.connect_to_keychain())
         await self.connection_established.wait()  # wait until connection is established.
 
     async def connect_to_keychain(self) -> None:
-        while not self.disconnect:
+        while not self.shut_down:
             try:
                 self.client_session = ClientSession()
                 self.websocket = await self.client_session.ws_connect(
@@ -135,7 +135,7 @@ class KeychainProxy(DaemonProxy):
         self.log.info("Close signal received from keychain, we probably timed out.")
 
     async def close(self) -> None:
-        self.disconnect = True
+        self.shut_down = True
         await super().close()
         if self.keychain_connection_task is not None:
             await self.keychain_connection_task
@@ -169,19 +169,17 @@ class KeychainProxy(DaemonProxy):
                     raise Exception(f"{err}")
                 raise Exception(f"{error}")
 
-    async def add_private_key(self, mnemonic: str, passphrase: str) -> PrivateKey:
+    async def add_private_key(self, mnemonic: str) -> PrivateKey:
         """
         Forwards to Keychain.add_private_key()
         """
         key: PrivateKey
         if self.use_local_keychain():
-            key = self.keychain.add_private_key(mnemonic, passphrase)
+            key = self.keychain.add_private_key(mnemonic)
         else:
-            response, success = await self.get_response_for_request(
-                "add_private_key", {"mnemonic": mnemonic, "passphrase": passphrase}
-            )
+            response, success = await self.get_response_for_request("add_private_key", {"mnemonic": mnemonic})
             if success:
-                seed = mnemonic_to_seed(mnemonic, passphrase)
+                seed = mnemonic_to_seed(mnemonic)
                 key = AugSchemeMPL.key_gen(seed)
             else:
                 error = response["data"].get("error", None)
@@ -254,7 +252,7 @@ class KeychainProxy(DaemonProxy):
                             continue  # We'll skip the incomplete key entry
                         ent = bytes.fromhex(ent_str)
                         mnemonic = bytes_to_mnemonic(ent)
-                        seed = mnemonic_to_seed(mnemonic, passphrase="")
+                        seed = mnemonic_to_seed(mnemonic)
                         key = AugSchemeMPL.key_gen(seed)
                         if bytes(key.get_g1()).hex() == pk:
                             keys.append((key, ent))
@@ -292,7 +290,7 @@ class KeychainProxy(DaemonProxy):
                         raise KeychainMalformedResponse(f"{err}")
                     ent = bytes.fromhex(ent_str)
                     mnemonic = bytes_to_mnemonic(ent)
-                    seed = mnemonic_to_seed(mnemonic, passphrase="")
+                    seed = mnemonic_to_seed(mnemonic)
                     sk = AugSchemeMPL.key_gen(seed)
                     if bytes(sk.get_g1()).hex() == pk:
                         key = sk
@@ -336,7 +334,7 @@ class KeychainProxy(DaemonProxy):
                     raise KeychainMalformedResponse(f"{err}")
                 else:
                     mnemonic = bytes_to_mnemonic(bytes.fromhex(ent))
-                    seed = mnemonic_to_seed(mnemonic, passphrase="")
+                    seed = mnemonic_to_seed(mnemonic)
                     private_key = AugSchemeMPL.key_gen(seed)
                     if bytes(private_key.get_g1()).hex() == pk:
                         key = private_key
@@ -381,7 +379,6 @@ async def connect_to_keychain(
 async def connect_to_keychain_and_validate(
     root_path: Path,
     log: logging.Logger,
-    *,
     user: Optional[str] = None,
     service: Optional[str] = None,
 ) -> Optional[KeychainProxy]:
@@ -410,5 +407,4 @@ async def connect_to_keychain_and_validate(
             return connection
     except Exception as e:
         print(f"Keychain(daemon) not started yet: {e}")
-        return None
     return None
