@@ -1,10 +1,14 @@
 import logging
 
-from blspy import PrivateKey
+from chinillablspy import PrivateKey
 from chinilla.cmds.init_funcs import check_keys
-from chinilla.util.keychain import Keychain
+from chinilla.util.errors import KeychainException, KeychainFingerprintNotFound
+from chinilla.util.ints import uint32
+from chinilla.util.keychain import KeyData, Keychain
+from chinilla.util.streamable import streamable, Streamable
 from pathlib import Path
-from typing import Any, Dict, List, Optional, cast
+from dataclasses import dataclass
+from typing import Any, Dict, List, Optional, Type
 
 # Commands that are handled by the KeychainServer
 keychain_commands = [
@@ -15,6 +19,10 @@ keychain_commands = [
     "get_all_private_keys",
     "get_first_private_key",
     "get_key_for_fingerprint",
+    "get_key",
+    "get_keys",
+    "set_label",
+    "delete_label",
 ]
 
 log = logging.getLogger(__name__)
@@ -26,6 +34,64 @@ KEYCHAIN_ERR_KEY_NOT_FOUND = "key not found"
 KEYCHAIN_ERR_MALFORMED_REQUEST = "malformed request"
 
 
+@streamable
+@dataclass(frozen=True)
+class EmptyResponse(Streamable):
+    pass
+
+
+@streamable
+@dataclass(frozen=True)
+class GetKeyResponse(Streamable):
+    key: KeyData
+
+
+@streamable
+@dataclass(frozen=True)
+class GetKeyRequest(Streamable):
+    fingerprint: uint32
+    include_secrets: bool = False
+
+    def run(self, keychain: Keychain) -> GetKeyResponse:
+        return GetKeyResponse(key=keychain.get_key(self.fingerprint, self.include_secrets))
+
+
+@streamable
+@dataclass(frozen=True)
+class GetKeysResponse(Streamable):
+    keys: List[KeyData]
+
+
+@streamable
+@dataclass(frozen=True)
+class GetKeysRequest(Streamable):
+    include_secrets: bool = False
+
+    def run(self, keychain: Keychain) -> GetKeysResponse:
+        return GetKeysResponse(keys=keychain.get_keys(self.include_secrets))
+
+
+@streamable
+@dataclass(frozen=True)
+class SetLabelRequest(Streamable):
+    fingerprint: uint32
+    label: str
+
+    def run(self, keychain: Keychain) -> EmptyResponse:
+        keychain.set_label(int(self.fingerprint), self.label)
+        return EmptyResponse()
+
+
+@streamable
+@dataclass(frozen=True)
+class DeleteLabelRequest(Streamable):
+    fingerprint: uint32
+
+    def run(self, keychain: Keychain) -> EmptyResponse:
+        keychain.delete_label(self.fingerprint)
+        return EmptyResponse()
+
+
 class KeychainServer:
     """
     Implements a remote keychain service for clients to perform key operations on
@@ -56,22 +122,30 @@ class KeychainServer:
                 self._alt_keychains[key] = keychain
         return keychain
 
-    async def handle_command(self, command, data) -> Dict[str, Any]:
+    async def handle_command(self, command: str, data: Dict[str, Any]) -> Dict[str, Any]:
         try:
             if command == "add_private_key":
-                return await self.add_private_key(cast(Dict[str, Any], data))
+                return await self.add_private_key(data)
             elif command == "check_keys":
-                return await self.check_keys(cast(Dict[str, Any], data))
+                return await self.check_keys(data)
             elif command == "delete_all_keys":
-                return await self.delete_all_keys(cast(Dict[str, Any], data))
+                return await self.delete_all_keys(data)
             elif command == "delete_key_by_fingerprint":
-                return await self.delete_key_by_fingerprint(cast(Dict[str, Any], data))
+                return await self.delete_key_by_fingerprint(data)
             elif command == "get_all_private_keys":
-                return await self.get_all_private_keys(cast(Dict[str, Any], data))
+                return await self.get_all_private_keys(data)
             elif command == "get_first_private_key":
-                return await self.get_first_private_key(cast(Dict[str, Any], data))
+                return await self.get_first_private_key(data)
             elif command == "get_key_for_fingerprint":
-                return await self.get_key_for_fingerprint(cast(Dict[str, Any], data))
+                return await self.get_key_for_fingerprint(data)
+            elif command == "get_key":
+                return await self.run_request(data, GetKeyRequest)
+            elif command == "get_keys":
+                return await self.run_request(data, GetKeysRequest)
+            elif command == "set_label":
+                return await self.run_request(data, SetLabelRequest)
+            elif command == "delete_label":
+                return await self.run_request(data, DeleteLabelRequest)
             return {}
         except Exception as e:
             log.exception(e)
@@ -82,16 +156,17 @@ class KeychainServer:
             return {"success": False, "error": KEYCHAIN_ERR_LOCKED}
 
         mnemonic = request.get("mnemonic", None)
-        passphrase = request.get("passphrase", None)
-        if mnemonic is None or passphrase is None:
+        label = request.get("label", None)
+
+        if mnemonic is None:
             return {
                 "success": False,
                 "error": KEYCHAIN_ERR_MALFORMED_REQUEST,
-                "error_details": {"message": "missing mnemonic and/or passphrase"},
+                "error_details": {"message": "missing mnemonic"},
             }
 
         try:
-            self.get_keychain_for_request(request).add_private_key(mnemonic, passphrase)
+            self.get_keychain_for_request(request).add_private_key(mnemonic, label)
         except KeyError as e:
             return {
                 "success": False,
@@ -147,6 +222,35 @@ class KeychainServer:
 
         return {"success": True}
 
+    async def run_request(self, request_dict: Dict[str, Any], request_type: Type[Any]) -> Dict[str, Any]:
+        keychain = self.get_keychain_for_request(request_dict)
+        if keychain.is_keyring_locked():
+            return {"success": False, "error": KEYCHAIN_ERR_LOCKED}
+
+        try:
+            request = request_type.from_json_dict(request_dict)
+        except Exception as e:
+            return {
+                "success": False,
+                "error": KEYCHAIN_ERR_MALFORMED_REQUEST,
+                "error_details": {"message": str(e)},
+            }
+
+        try:
+            return {"success": True, **request.run(keychain).to_json_dict()}
+        except KeychainFingerprintNotFound as e:
+            return {
+                "success": False,
+                "error": KEYCHAIN_ERR_KEY_NOT_FOUND,
+                "error_details": {"fingerprint": e.fingerprint},
+            }
+        except KeychainException as e:
+            return {
+                "success": False,
+                "error": KEYCHAIN_ERR_MALFORMED_REQUEST,
+                "error_details": {"message": str(e)},
+            }
+
     async def get_all_private_keys(self, request: Dict[str, Any]) -> Dict[str, Any]:
         all_keys: List[Dict[str, Any]] = []
         if self.get_keychain_for_request(request).is_keyring_locked():
