@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import asyncio
 import json
 import logging
@@ -7,7 +9,7 @@ from pathlib import Path
 from typing import Any, Callable, Dict, List, Optional, Set, Tuple
 
 import aiohttp
-from blspy import AugSchemeMPL, G1Element, G2Element, PrivateKey
+from chinillablspy import AugSchemeMPL, G1Element, G2Element, PrivateKey
 
 import chinilla.server.ws_connection as ws  # lgtm [py/import-and-import-from]
 from chinilla.consensus.constants import ConsensusConstants
@@ -28,6 +30,7 @@ from chinilla.protocols.pool_protocol import (
     get_current_authentication_token,
 )
 from chinilla.protocols.protocol_message_types import ProtocolMessageTypes
+from chinilla.rpc.rpc_server import default_get_connections
 from chinilla.server.outbound_message import NodeType, make_msg
 from chinilla.server.server import ssl_context_for_root
 from chinilla.server.ws_connection import WSChinillaConnection
@@ -114,6 +117,9 @@ class Farmer:
         # Last time we updated pool_state based on the config file
         self.last_config_access_time: uint64 = uint64(0)
 
+    def get_connections(self, request_node_type: Optional[NodeType]) -> List[Dict[str, Any]]:
+        return default_get_connections(server=self.server, request_node_type=request_node_type)
+
     async def ensure_keychain_proxy(self) -> KeychainProxy:
         if self.keychain_proxy is None:
             if self.local_keychain:
