@@ -1,9 +1,11 @@
+from __future__ import annotations
+
 import json
 import time
 from typing import Any, Dict, List, Optional, Tuple
 
 import aiohttp
-from blspy import AugSchemeMPL, G2Element, PrivateKey
+from chinillablspy import AugSchemeMPL, G2Element, PrivateKey
 
 import chinilla.server.ws_connection as ws
 from chinilla import __version__
