@@ -1,11 +1,13 @@
+from __future__ import annotations
+
 import logging
 from collections import Counter
 from pathlib import Path
 from time import sleep, time
 from typing import List
 
-from blspy import G1Element
-from chiapos import Verifier
+from chinillablspy import G1Element
+from chinillapos import Verifier
 
 from chinilla.plotting.manager import PlotManager
 from chinilla.plotting.util import (
