@@ -1,11 +1,13 @@
+from __future__ import annotations
+
 import logging
 from datetime import datetime
 from pathlib import Path
 from secrets import token_bytes
 from typing import Dict, List, Optional, Tuple
 
-from blspy import AugSchemeMPL, G1Element, PrivateKey
-from chiapos import DiskPlotter
+from chinillablspy import AugSchemeMPL, G1Element, PrivateKey
+from chinillapos import DiskPlotter
 
 from chinilla.daemon.keychain_proxy import KeychainProxy, connect_to_keychain_and_validate, wrap_local_keychain
 from chinilla.plotting.util import stream_plot_info_ph, stream_plot_info_pk
@@ -208,10 +210,6 @@ async def create_plots(
             log.info(f"Debug memo: {args.memo}")
             plot_memo = bytes32.fromhex(args.memo)
 
-        # Uncomment next two lines if memo is needed for dev debug
-        plot_memo_str: str = plot_memo.hex()
-        log.info(f"Memo: {plot_memo_str}")
-
         dt_string = datetime.now().strftime("%Y-%m-%d-%H-%M")
 
         if use_datetime:
