@@ -1,9 +1,12 @@
+from __future__ import annotations
+
 from typing import Optional
 
 import chinilla.server.ws_connection as ws
-from chinilla.full_node.full_node import full_node_protocol, wallet_protocol
+from chinilla.protocols import full_node_protocol, wallet_protocol
 from chinilla.seeder.crawler import Crawler
 from chinilla.server.outbound_message import Message
+from chinilla.server.server import ChinillaServer
 from chinilla.util.api_decorators import api_request, peer_required
 
 
@@ -20,7 +23,8 @@ class CrawlerAPI:
         return invoke
 
     @property
-    def server(self):
+    def server(self) -> ChinillaServer:
+        assert self.crawler.server is not None
         return self.crawler.server
 
     @property
