@@ -7,7 +7,7 @@ from chinilla.types.peer_info import PeerInfo
 from chinilla.util.network import get_host_addr
 
 
-def start_reconnect_task(server: ChinillaServer, peer_info_arg: PeerInfo, log, auth: bool, prefer_ipv6: Optional[bool]):
+def start_reconnect_task(server: ChinillaServer, peer_info_arg: PeerInfo, log, prefer_ipv6: Optional[bool]):
     """
     Start a background task that checks connection and reconnects periodically to a peer.
     """
@@ -26,7 +26,7 @@ def start_reconnect_task(server: ChinillaServer, peer_info_arg: PeerInfo, log, a
             if peer_retry:
                 log.info(f"Reconnecting to peer {peer_info}")
                 try:
-                    await server.start_client(peer_info, None, auth=auth)
+                    await server.start_client(peer_info, None)
                 except Exception as e:
                     log.info(f"Failed to connect to {peer_info} {e}")
             await asyncio.sleep(3)
