@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import pathlib
 import sys
 from typing import Dict, Optional
@@ -10,7 +12,7 @@ from chinilla.rpc.harvester_rpc_api import HarvesterRpcApi
 from chinilla.server.outbound_message import NodeType
 from chinilla.server.start_service import RpcInfo, Service, async_run
 from chinilla.types.peer_info import PeerInfo
-from chinilla.util.chinilla_logging import initialize_logging
+from chinilla.util.chinilla_logging import initialize_service_logging
 from chinilla.util.config import load_config, load_config_cli
 from chinilla.util.default_root import DEFAULT_ROOT_PATH
 
@@ -24,11 +26,11 @@ def create_harvester_service(
     root_path: pathlib.Path,
     config: Dict,
     consensus_constants: ConsensusConstants,
+    farmer_peer: Optional[PeerInfo],
     connect_to_daemon: bool = True,
-) -> Service:
+) -> Service[Harvester]:
     service_config = config[SERVICE_NAME]
 
-    connect_peers = [PeerInfo(service_config["farmer_peer"]["host"], service_config["farmer_peer"]["port"])]
     overrides = service_config["network_overrides"]["constants"][service_config["selected_network"]]
     updated_constants = consensus_constants.replace_str_to_bytes(**overrides)
 
@@ -47,8 +49,7 @@ def create_harvester_service(
         advertised_port=service_config["port"],
         service_name=SERVICE_NAME,
         server_listen_ports=[service_config["port"]],
-        connect_peers=connect_peers,
-        auth_connect_peers=True,
+        connect_peers=[] if farmer_peer is None else [farmer_peer],
         network_id=network_id,
         rpc_info=rpc_info,
         connect_to_daemon=connect_to_daemon,
@@ -60,12 +61,9 @@ async def async_main() -> int:
     config = load_config(DEFAULT_ROOT_PATH, "config.yaml")
     service_config = load_config_cli(DEFAULT_ROOT_PATH, "config.yaml", SERVICE_NAME)
     config[SERVICE_NAME] = service_config
-    initialize_logging(
-        service_name=SERVICE_NAME,
-        logging_config=service_config["logging"],
-        root_path=DEFAULT_ROOT_PATH,
-    )
-    service = create_harvester_service(DEFAULT_ROOT_PATH, config, DEFAULT_CONSTANTS)
+    initialize_service_logging(service_name=SERVICE_NAME, config=config)
+    farmer_peer = PeerInfo(service_config["farmer_peer"]["host"], service_config["farmer_peer"]["port"])
+    service = create_harvester_service(DEFAULT_ROOT_PATH, config, DEFAULT_CONSTANTS, farmer_peer)
     await service.setup_process_global_state()
     await service.run()
 
