@@ -1,4 +1,5 @@
 import pathlib
+import os
 from multiprocessing import freeze_support
 import sys
 from typing import Dict, Optional
@@ -9,7 +10,7 @@ from chinilla.rpc.wallet_rpc_api import WalletRpcApi
 from chinilla.server.outbound_message import NodeType
 from chinilla.server.start_service import RpcInfo, Service, async_run
 from chinilla.types.peer_info import PeerInfo
-from chinilla.util.chinilla_logging import initialize_logging
+from chinilla.util.chinilla_logging import initialize_service_logging
 from chinilla.util.config import load_config_cli, load_config
 from chinilla.util.default_root import DEFAULT_ROOT_PATH
 from chinilla.util.keychain import Keychain
@@ -29,7 +30,7 @@ def create_wallet_service(
     consensus_constants: ConsensusConstants,
     keychain: Optional[Keychain] = None,
     connect_to_daemon: bool = True,
-) -> Service:
+) -> Service[WalletNode]:
     service_config = config[SERVICE_NAME]
 
     overrides = service_config["network_overrides"]["constants"][service_config["selected_network"]]
@@ -73,7 +74,6 @@ def create_wallet_service(
         service_name=SERVICE_NAME,
         on_connect_callback=node.on_connect,
         connect_peers=connect_peers,
-        auth_connect_peers=False,
         network_id=network_id,
         rpc_info=rpc_info,
         advertised_port=service_config["port"],
@@ -98,11 +98,7 @@ async def async_main() -> int:
         service_config["selected_network"] = "testnet0"
     else:
         constants = DEFAULT_CONSTANTS
-    initialize_logging(
-        service_name=SERVICE_NAME,
-        logging_config=service_config["logging"],
-        root_path=DEFAULT_ROOT_PATH,
-    )
+    initialize_service_logging(service_name=SERVICE_NAME, config=config)
     service = create_wallet_service(DEFAULT_ROOT_PATH, config, constants)
     await service.setup_process_global_state()
     await service.run()
@@ -112,6 +108,12 @@ async def async_main() -> int:
 
 def main() -> int:
     freeze_support()
+    if os.getenv("CHINILLA_INSTRUMENT_WALLET", 0) != 0:
+        from chinilla.util.task_timing import start_task_instrumentation, stop_task_instrumentation
+        import atexit
+
+        start_task_instrumentation()
+        atexit.register(stop_task_instrumentation)
     return async_run(async_main())
 
 
