@@ -1,3 +1,4 @@
+from __future__ import annotations
 import io
 from typing import Callable, Dict, List, Set, Tuple, Optional, Any
 
@@ -5,8 +6,7 @@ from clvm import SExp
 from clvm.casts import int_from_bytes
 from clvm.EvalError import EvalError
 from clvm.serialize import sexp_from_stream, sexp_to_stream
-from chia_rs import MEMPOOL_MODE, run_chia_program, serialized_length, run_generator
-from clvm_tools.curry import uncurry
+from chinilla_rs import MEMPOOL_MODE, run_chinilla_program, serialized_length, run_generator, tree_hash
 
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.util.hash import std_hash
@@ -32,11 +32,18 @@ class Program(SExp):
         sexp_to_stream(self, f)
 
     @classmethod
-    def from_bytes(cls, blob: bytes) -> "Program":
-        f = io.BytesIO(blob)
-        result = cls.parse(f)  # noqa
-        assert f.read() == b""
-        return result
+    def from_bytes(cls, blob: bytes) -> Program:
+        # this runs the program "1", which just returns the first argument.
+        # the first argument is the buffer we want to parse. This effectively
+        # leverages the rust parser and LazyNode, making it a lot faster to
+        # parse serialized programs into a python compatible structure
+        cost, ret = run_chinilla_program(
+            b"\x01",
+            blob,
+            50,
+            0,
+        )
+        return Program.to(ret)
 
     @classmethod
     def fromhex(cls, hexstr: str) -> "Program":
@@ -95,16 +102,19 @@ class Program(SExp):
         """
         return _sexp_replace(self, self.to, **kwargs)
 
-    def get_tree_hash(self, *args: bytes32) -> bytes32:
+    def get_tree_hash_precalc(self, *args: bytes32) -> bytes32:
         """
         Any values in `args` that appear in the tree
         are presumed to have been hashed already.
         """
         return sha256_treehash(self, set(args))
 
+    def get_tree_hash(self) -> bytes32:
+        return bytes32(tree_hash(bytes(self)))
+
     def run_with_cost(self, max_cost: int, args) -> Tuple[int, "Program"]:
         prog_args = Program.to(args)
-        cost, r = run_chia_program(self.as_bin(), prog_args.as_bin(), max_cost, 0)
+        cost, r = run_chinilla_program(self.as_bin(), prog_args.as_bin(), max_cost, 0)
         return cost, Program.to(r)
 
     def run(self, args) -> "Program":
@@ -133,11 +143,37 @@ class Program(SExp):
             fixed_args = [4, (1, arg), fixed_args]
         return Program.to([2, (1, self), fixed_args])
 
-    def uncurry(self) -> Tuple["Program", "Program"]:
-        r = uncurry(self)
-        if r is None:
+    def uncurry(self) -> Tuple[Program, Program]:
+        def match(o: SExp, expected: bytes) -> None:
+            if o.atom != expected:
+                raise ValueError(f"expected: {expected.hex()}")
+
+        try:
+            # (2 (1 . <mod>) <args>)
+            ev, quoted_inner, args_list = self.as_iter()
+            match(ev, b"\x02")
+            match(quoted_inner.pair[0], b"\x01")
+            mod = quoted_inner.pair[1]
+            args = []
+            while args_list.pair is not None:
+                # (4 (1 . <arg>) <rest>)
+                cons, quoted_arg, rest = args_list.as_iter()
+                match(cons, b"\x04")
+                match(quoted_arg.pair[0], b"\x01")
+                args.append(quoted_arg.pair[1])
+                args_list = rest
+            match(args_list, b"\x01")
+            return Program.to(mod), Program.to(args)
+        except ValueError:  # too many values to unpack
+            # when unpacking as_iter()
+            # or when a match() fails
+            return self, self.to(0)
+        except TypeError:  # NoneType not subscriptable
+            # when an object is not a pair or atom as expected
+            return self, self.to(0)
+        except EvalError:  # first of non-cons
+            # when as_iter() fails
             return self, self.to(0)
-        return r
 
     def as_int(self) -> int:
         return int_from_bytes(self.as_atom())
@@ -249,13 +285,8 @@ class SerializedProgram:
             return True
         return self._buf != other._buf
 
-    def get_tree_hash(self, *args: bytes32) -> bytes32:
-        """
-        Any values in `args` that appear in the tree
-        are presumed to have been hashed already.
-        """
-        tmp = sexp_from_stream(io.BytesIO(self._buf), SExp.to)
-        return _tree_hash(tmp, set(args))
+    def get_tree_hash(self) -> bytes32:
+        return bytes32(tree_hash(self._buf))
 
     def run_mempool_with_cost(self, max_cost: int, *args) -> Tuple[int, Program]:
         return self._run(max_cost, MEMPOOL_MODE, *args)
@@ -263,13 +294,13 @@ class SerializedProgram:
     def run_with_cost(self, max_cost: int, *args) -> Tuple[int, Program]:
         return self._run(max_cost, 0, *args)
 
-    # returns an optional error code and an optional SpendBundleConditions (from chia_rs)
+    # returns an optional error code and an optional SpendBundleConditions (from chinilla_rs)
     # exactly one of those will hold a value
     def run_as_generator(
         self, max_cost: int, flags: int, *args
     ) -> Tuple[Optional[int], Optional[SpendBundleConditions]]:
 
-        serialized_args = b""
+        serialized_args = bytearray()
         if len(args) > 1:
             # when we have more than one argument, serialize them into a list
             for a in args:
@@ -281,7 +312,7 @@ class SerializedProgram:
 
         err, ret = run_generator(
             self._buf,
-            serialized_args,
+            bytes(serialized_args),
             max_cost,
             flags,
         )
@@ -297,7 +328,7 @@ class SerializedProgram:
         # buffer. Some arguments may already be in serialized form (e.g.
         # SerializedProgram) so we don't want to de-serialize those just to
         # serialize them back again. This is handled by _serialize()
-        serialized_args = b""
+        serialized_args = bytearray()
         if len(args) > 1:
             # when we have more than one argument, serialize them into a list
             for a in args:
@@ -307,9 +338,9 @@ class SerializedProgram:
         else:
             serialized_args += _serialize(args[0])
 
-        cost, ret = run_chia_program(
+        cost, ret = run_chinilla_program(
             self._buf,
-            serialized_args,
+            bytes(serialized_args),
             max_cost,
             flags,
         )
