@@ -6,12 +6,12 @@ import yaml
 
 from chinilla.cmds.passphrase_funcs import prompt_for_passphrase, read_passphrase_from_file
 from chinilla.util.default_root import DEFAULT_KEYS_ROOT_PATH
-from chinilla.util.file_keyring import FileKeyring
+from chinilla.util.file_keyring import FileKeyringContent
 from chinilla.util.keyring_wrapper import DEFAULT_PASSPHRASE_IF_NO_MASTER_PASSPHRASE, KeyringWrapper
 from cryptography.exceptions import InvalidTag
 from io import TextIOWrapper
 from pathlib import Path
-from typing import Any, Dict, Optional
+from typing import Optional
 
 DEFAULT_KEYRING_YAML = DEFAULT_KEYS_ROOT_PATH / "keyring.yaml"
 
@@ -44,7 +44,6 @@ def dump(keyring_file, full_payload: bool, passphrase_file: Optional[TextIOWrapp
     saved_passphrase: Optional[str] = KeyringWrapper.get_shared_instance().get_master_passphrase_from_credential_store()
     passphrase: str = saved_passphrase or DEFAULT_PASSPHRASE_IF_NO_MASTER_PASSPHRASE
     prompt: str = get_passphrase_prompt(str(keyring_file))
-    data: Dict[str, Any] = {}
 
     print(f"Attempting to dump contents of keyring file: {keyring_file}\n")
 
@@ -52,24 +51,22 @@ def dump(keyring_file, full_payload: bool, passphrase_file: Optional[TextIOWrapp
         passphrase = read_passphrase_from_file(passphrase_file)
 
     keyring_path = Path(keyring_file)
-    keyring = FileKeyring(keyring_path)
-
-    if full_payload:
-        keyring.load_outer_payload()
-        data = keyring.outer_payload_cache
+    file_content = FileKeyringContent.create_from_path(keyring_path)
+    file_content_dict = file_content.to_dict()
 
     for i in range(5):
         try:
-            keyring.load_keyring(passphrase)
-            if len(data) > 0:
-                data["data"] = keyring.payload_cache
+            data_dict = file_content.get_decrypted_data_dict(passphrase)
+            if full_payload:
+                dump_content = file_content_dict
+                dump_content["data"] = data_dict
             else:
-                data = keyring.payload_cache
+                dump_content = data_dict
 
             if pretty_print:
-                print(yaml.dump(data))
-            else:
-                print(data)
+                dump_content = yaml.dump(dump_content)
+
+            print(dump_content)
             break
         except (ValueError, InvalidTag):
             passphrase = prompt_for_passphrase(prompt)
