@@ -1,29 +1,35 @@
-import colorama
+from __future__ import annotations
 import pkg_resources
 import sys
 import unicodedata
 
 from bitstring import BitArray  # pyright: reportMissingImports=false
-from blspy import AugSchemeMPL, G1Element, PrivateKey  # pyright: reportMissingImports=false
-from chinilla.util.errors import KeychainNotSet, KeychainMaxUnlockAttempts
+from chinillablspy import AugSchemeMPL, G1Element, PrivateKey  # pyright: reportMissingImports=false
+from chinilla.util.errors import (
+    KeychainException,
+    KeychainNotSet,
+    KeychainKeyDataMismatch,
+    KeychainFingerprintExists,
+    KeychainFingerprintNotFound,
+    KeychainSecretsMissing,
+    KeychainUserNotFound,
+)
 from chinilla.util.hash import std_hash
+from chinilla.util.ints import uint32
 from chinilla.util.keyring_wrapper import KeyringWrapper
+from chinilla.util.streamable import streamable, Streamable
+from dataclasses import dataclass
 from hashlib import pbkdf2_hmac
 from pathlib import Path
 from secrets import token_bytes
-from time import sleep
 from typing import Any, Dict, List, Optional, Tuple
 
+from typing_extensions import final
 
 CURRENT_KEY_VERSION = "1.8"
 DEFAULT_USER = f"user-chinilla-{CURRENT_KEY_VERSION}"  # e.g. user-chinilla-1.8
 DEFAULT_SERVICE = f"chinilla-{DEFAULT_USER}"  # e.g. chinilla-user-chinilla-1.8
-DEFAULT_PASSPHRASE_PROMPT = (
-    colorama.Fore.YELLOW + colorama.Style.BRIGHT + "(Unlock Keyring)" + colorama.Style.RESET_ALL + " Passphrase: "
-)  # noqa: E501
-FAILED_ATTEMPT_DELAY = 0.5
 MAX_KEYS = 100
-MAX_RETRIES = 3
 MIN_PASSPHRASE_LEN = 8
 
 
@@ -45,67 +51,6 @@ def set_keys_root_path(keys_root_path: Path) -> None:
     KeyringWrapper.set_keys_root_path(keys_root_path)
 
 
-def obtain_current_passphrase(prompt: str = DEFAULT_PASSPHRASE_PROMPT, use_passphrase_cache: bool = False) -> str:
-    """
-    Obtains the master passphrase for the keyring, optionally using the cached
-    value (if previously set). If the passphrase isn't already cached, the user is
-    prompted interactively to enter their passphrase a max of MAX_RETRIES times
-    before failing.
-    """
-    from chinilla.cmds.passphrase_funcs import prompt_for_passphrase
-
-    if use_passphrase_cache:
-        passphrase, validated = KeyringWrapper.get_shared_instance().get_cached_master_passphrase()
-        if passphrase:
-            # If the cached passphrase was previously validated, we assume it's... valid
-            if validated:
-                return passphrase
-
-            # Cached passphrase needs to be validated
-            if KeyringWrapper.get_shared_instance().master_passphrase_is_valid(passphrase):
-                KeyringWrapper.get_shared_instance().set_cached_master_passphrase(passphrase, validated=True)
-                return passphrase
-            else:
-                # Cached passphrase is bad, clear the cache
-                KeyringWrapper.get_shared_instance().set_cached_master_passphrase(None)
-
-    # Prompt interactively with up to MAX_RETRIES attempts
-    for i in range(MAX_RETRIES):
-        colorama.init()
-
-        passphrase = prompt_for_passphrase(prompt)
-
-        if KeyringWrapper.get_shared_instance().master_passphrase_is_valid(passphrase):
-            # If using the passphrase cache, and the user inputted a passphrase, update the cache
-            if use_passphrase_cache:
-                KeyringWrapper.get_shared_instance().set_cached_master_passphrase(passphrase, validated=True)
-            return passphrase
-
-        sleep(FAILED_ATTEMPT_DELAY)
-        print("Incorrect passphrase\n")
-    raise KeychainMaxUnlockAttempts()
-
-
-def unlocks_keyring(use_passphrase_cache=False):
-    """
-    Decorator used to unlock the keyring interactively, if necessary
-    """
-
-    def inner(func):
-        def wrapper(*args, **kwargs):
-            try:
-                if KeyringWrapper.get_shared_instance().has_master_passphrase():
-                    obtain_current_passphrase(use_passphrase_cache=use_passphrase_cache)
-            except Exception as e:
-                print(f"Unable to unlock the keyring: {e}")
-                sys.exit(1)
-            return func(*args, **kwargs)
-
-        return wrapper
-
-    return inner
-
-
 def bip39_word_list() -> str:
     return pkg_resources.resource_string(__name__, "english.txt").decode()
 
@@ -172,11 +117,11 @@ def bytes_from_mnemonic(mnemonic_str: str) -> bytes:
     return entropy_bytes
 
 
-def mnemonic_to_seed(mnemonic: str, passphrase: str) -> bytes:
+def mnemonic_to_seed(mnemonic: str) -> bytes:
     """
     Uses BIP39 standard to derive a seed from entropy bytes.
     """
-    salt_str: str = "mnemonic" + passphrase
+    salt_str: str = "mnemonic"
     salt = unicodedata.normalize("NFKD", salt_str).encode("utf-8")
     mnemonic_normalized = unicodedata.normalize("NFKD", mnemonic).encode("utf-8")
     seed = pbkdf2_hmac("sha512", mnemonic_normalized, salt, 2048)
@@ -200,9 +145,110 @@ def get_private_key_user(user: str, index: int) -> str:
     return f"wallet-{user}-{index}"
 
 
+@final
+@streamable
+@dataclass(frozen=True)
+class KeyDataSecrets(Streamable):
+    mnemonic: List[str]
+    entropy: bytes
+    private_key: PrivateKey
+
+    def __post_init__(self) -> None:
+        # This is redundant if `from_*` methods are used but its to make sure there can't be an `KeyDataSecrets`
+        # instance with an attribute mismatch for calculated cached values. Should be ok since we don't handle a lot of
+        # keys here.
+        mnemonic_str = self.mnemonic_str()
+        try:
+            bytes_from_mnemonic(mnemonic_str)
+        except Exception as e:
+            raise KeychainKeyDataMismatch("mnemonic") from e
+        if bytes_from_mnemonic(mnemonic_str) != self.entropy:
+            raise KeychainKeyDataMismatch("entropy")
+        if AugSchemeMPL.key_gen(mnemonic_to_seed(mnemonic_str)) != self.private_key:
+            raise KeychainKeyDataMismatch("private_key")
+
+    @classmethod
+    def from_mnemonic(cls, mnemonic: str) -> KeyDataSecrets:
+        return cls(
+            mnemonic=mnemonic.split(),
+            entropy=bytes_from_mnemonic(mnemonic),
+            private_key=AugSchemeMPL.key_gen(mnemonic_to_seed(mnemonic)),
+        )
+
+    @classmethod
+    def from_entropy(cls, entropy: bytes) -> KeyDataSecrets:
+        return cls.from_mnemonic(bytes_to_mnemonic(entropy))
+
+    @classmethod
+    def generate(cls) -> KeyDataSecrets:
+        return cls.from_mnemonic(generate_mnemonic())
+
+    def mnemonic_str(self) -> str:
+        return " ".join(self.mnemonic)
+
+
+@final
+@streamable
+@dataclass(frozen=True)
+class KeyData(Streamable):
+    fingerprint: uint32
+    public_key: G1Element
+    label: Optional[str]
+    secrets: Optional[KeyDataSecrets]
+
+    def __post_init__(self) -> None:
+        # This is redundant if `from_*` methods are used but its to make sure there can't be an `KeyData` instance with
+        # an attribute mismatch for calculated cached values. Should be ok since we don't handle a lot of keys here.
+        if self.secrets is not None and self.public_key != self.private_key.get_g1():
+            raise KeychainKeyDataMismatch("public_key")
+        if self.public_key.get_fingerprint() != self.fingerprint:
+            raise KeychainKeyDataMismatch("fingerprint")
+
+    @classmethod
+    def from_mnemonic(cls, mnemonic: str, label: Optional[str] = None) -> KeyData:
+        private_key = AugSchemeMPL.key_gen(mnemonic_to_seed(mnemonic))
+        return cls(
+            fingerprint=private_key.get_g1().get_fingerprint(),
+            public_key=private_key.get_g1(),
+            label=label,
+            secrets=KeyDataSecrets.from_mnemonic(mnemonic),
+        )
+
+    @classmethod
+    def from_entropy(cls, entropy: bytes, label: Optional[str] = None) -> KeyData:
+        return cls.from_mnemonic(bytes_to_mnemonic(entropy), label)
+
+    @classmethod
+    def generate(cls, label: Optional[str] = None) -> KeyData:
+        return cls.from_mnemonic(generate_mnemonic(), label)
+
+    @property
+    def mnemonic(self) -> List[str]:
+        if self.secrets is None:
+            raise KeychainSecretsMissing()
+        return self.secrets.mnemonic
+
+    def mnemonic_str(self) -> str:
+        if self.secrets is None:
+            raise KeychainSecretsMissing()
+        return self.secrets.mnemonic_str()
+
+    @property
+    def entropy(self) -> bytes:
+        if self.secrets is None:
+            raise KeychainSecretsMissing()
+        return self.secrets.entropy
+
+    @property
+    def private_key(self) -> PrivateKey:
+        if self.secrets is None:
+            raise KeychainSecretsMissing()
+        return self.secrets.private_key
+
+
 class Keychain:
     """
-    The keychain stores two types of keys: private keys, which are PrivateKeys from blspy,
+    The keychain stores two types of keys: private keys, which are PrivateKeys from chinillablspy,
     and private key seeds, which are bytes objects that are used as a seed to construct
     PrivateKeys. Private key seeds are converted to mnemonics when shown to users.
 
@@ -224,19 +270,26 @@ class Keychain:
 
         self.keyring_wrapper = keyring_wrapper
 
-    def _get_pk_and_entropy(self, user: str) -> Optional[Tuple[G1Element, bytes]]:
+    def _get_key_data(self, index: int, include_secrets: bool = True) -> KeyData:
         """
-        Returns the keychain contents for a specific 'user' (key index). The contents
-        include an G1Element and the entropy required to generate the private key.
-        Note that generating the actual private key also requires the passphrase.
+        Returns the parsed keychain contents for a specific 'user' (key index). The content
+        is represented by the class `KeyData`.
         """
+        user = get_private_key_user(self.user, index)
         read_str = self.keyring_wrapper.get_passphrase(self.service, user)
         if read_str is None or len(read_str) == 0:
-            return None
+            raise KeychainUserNotFound(self.service, user)
         str_bytes = bytes.fromhex(read_str)
-        return (
-            G1Element.from_bytes(str_bytes[: G1Element.SIZE]),
-            str_bytes[G1Element.SIZE :],  # flake8: noqa
+
+        public_key = G1Element.from_bytes(str_bytes[: G1Element.SIZE])
+        fingerprint = public_key.get_fingerprint()
+        entropy = str_bytes[G1Element.SIZE : G1Element.SIZE + 32]
+
+        return KeyData(
+            fingerprint=fingerprint,
+            public_key=public_key,
+            label=self.keyring_wrapper.get_label(fingerprint),
+            secrets=KeyDataSecrets.from_entropy(entropy) if include_secrets else None,
         )
 
     def _get_free_private_key_index(self) -> int:
@@ -245,20 +298,19 @@ class Keychain:
         """
         index = 0
         while True:
-            pk = get_private_key_user(self.user, index)
-            pkent = self._get_pk_and_entropy(pk)
-            if pkent is None:
+            try:
+                self._get_key_data(index)
+                index += 1
+            except KeychainUserNotFound:
                 return index
-            index += 1
 
-    @unlocks_keyring(use_passphrase_cache=True)
-    def add_private_key(self, mnemonic: str, passphrase: str) -> PrivateKey:
+    def add_private_key(self, mnemonic: str, label: Optional[str] = None) -> PrivateKey:
         """
         Adds a private key to the keychain, with the given entropy and passphrase. The
         keychain itself will store the public key, and the entropy bytes,
         but not the passphrase.
         """
-        seed = mnemonic_to_seed(mnemonic, passphrase)
+        seed = mnemonic_to_seed(mnemonic)
         entropy = bytes_from_mnemonic(mnemonic)
         index = self._get_free_private_key_index()
         key = AugSchemeMPL.key_gen(seed)
@@ -266,181 +318,163 @@ class Keychain:
 
         if fingerprint in [pk.get_fingerprint() for pk in self.get_all_public_keys()]:
             # Prevents duplicate add
-            return key
+            raise KeychainFingerprintExists(fingerprint)
+
+        # Try to set the label first, it may fail if the label is invalid or already exists.
+        # This can probably just be moved into `FileKeyring.set_passphrase` after the legacy keyring stuff was dropped.
+        if label is not None:
+            self.keyring_wrapper.set_label(fingerprint, label)
+
+        try:
+            self.keyring_wrapper.set_passphrase(
+                self.service,
+                get_private_key_user(self.user, index),
+                bytes(key.get_g1()).hex() + entropy.hex(),
+            )
+        except Exception:
+            if label is not None:
+                self.keyring_wrapper.delete_label(fingerprint)
+            raise
 
-        self.keyring_wrapper.set_passphrase(
-            self.service,
-            get_private_key_user(self.user, index),
-            bytes(key.get_g1()).hex() + entropy.hex(),
-        )
         return key
 
-    def get_first_private_key(self, passphrases: List[str] = [""]) -> Optional[Tuple[PrivateKey, bytes]]:
+    def set_label(self, fingerprint: int, label: str) -> None:
+        """
+        Assigns the given label to the first key with the given fingerprint.
+        """
+        self.get_key(fingerprint)  # raise if the fingerprint doesn't exist
+        self.keyring_wrapper.set_label(fingerprint, label)
+
+    def delete_label(self, fingerprint: int) -> None:
+        """
+        Removes the label assigned to the key with the given fingerprint.
+        """
+        self.keyring_wrapper.delete_label(fingerprint)
+
+    def get_first_private_key(self) -> Optional[Tuple[PrivateKey, bytes]]:
         """
         Returns the first key in the keychain that has one of the passed in passphrases.
         """
-        index = 0
-        pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
-        while index <= MAX_KEYS:
-            if pkent is not None:
-                pk, ent = pkent
-                for pp in passphrases:
-                    mnemonic = bytes_to_mnemonic(ent)
-                    seed = mnemonic_to_seed(mnemonic, pp)
-                    key = AugSchemeMPL.key_gen(seed)
-                    if key.get_g1() == pk:
-                        return (key, ent)
-            index += 1
-            pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
+        for index in range(MAX_KEYS + 1):
+            try:
+                key_data = self._get_key_data(index)
+                return key_data.private_key, key_data.entropy
+            except KeychainUserNotFound:
+                pass
         return None
 
-    def get_private_key_by_fingerprint(
-        self, fingerprint: int, passphrases: List[str] = [""]
-    ) -> Optional[Tuple[PrivateKey, bytes]]:
+    def get_private_key_by_fingerprint(self, fingerprint: int) -> Optional[Tuple[PrivateKey, bytes]]:
         """
         Return first private key which have the given public key fingerprint.
         """
-        index = 0
-        pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
-        while index <= MAX_KEYS:
-            if pkent is not None:
-                pk, ent = pkent
-                for pp in passphrases:
-                    mnemonic = bytes_to_mnemonic(ent)
-                    seed = mnemonic_to_seed(mnemonic, pp)
-                    key = AugSchemeMPL.key_gen(seed)
-                    if pk.get_fingerprint() == fingerprint:
-                        return (key, ent)
-            index += 1
-            pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
+        for index in range(MAX_KEYS + 1):
+            try:
+                key_data = self._get_key_data(index)
+                if key_data.fingerprint == fingerprint:
+                    return key_data.private_key, key_data.entropy
+            except KeychainUserNotFound:
+                pass
         return None
 
-    def get_all_private_keys(self, passphrases: List[str] = [""]) -> List[Tuple[PrivateKey, bytes]]:
+    def get_all_private_keys(self) -> List[Tuple[PrivateKey, bytes]]:
         """
         Returns all private keys which can be retrieved, with the given passphrases.
         A tuple of key, and entropy bytes (i.e. mnemonic) is returned for each key.
         """
         all_keys: List[Tuple[PrivateKey, bytes]] = []
+        for index in range(MAX_KEYS + 1):
+            try:
+                key_data = self._get_key_data(index)
+                all_keys.append((key_data.private_key, key_data.entropy))
+            except KeychainUserNotFound:
+                pass
+        return all_keys
 
-        index = 0
-        pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
-        while index <= MAX_KEYS:
-            if pkent is not None:
-                pk, ent = pkent
-                for pp in passphrases:
-                    mnemonic = bytes_to_mnemonic(ent)
-                    seed = mnemonic_to_seed(mnemonic, pp)
-                    key = AugSchemeMPL.key_gen(seed)
-                    if key.get_g1() == pk:
-                        all_keys.append((key, ent))
-            index += 1
-            pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
+    def get_key(self, fingerprint: int, include_secrets: bool = False) -> KeyData:
+        """
+        Return the KeyData of the first key which has the given public key fingerprint.
+        """
+        for index in range(MAX_KEYS + 1):
+            try:
+                key_data = self._get_key_data(index, include_secrets)
+                if key_data.public_key.get_fingerprint() == fingerprint:
+                    return key_data
+            except KeychainUserNotFound:
+                pass
+        raise KeychainFingerprintNotFound(fingerprint)
+
+    def get_keys(self, include_secrets: bool = False) -> List[KeyData]:
+        """
+        Returns the KeyData of all keys which can be retrieved.
+        """
+        all_keys: List[KeyData] = []
+        for index in range(MAX_KEYS + 1):
+            try:
+                key_data = self._get_key_data(index, include_secrets)
+                all_keys.append(key_data)
+            except KeychainUserNotFound:
+                pass
         return all_keys
 
     def get_all_public_keys(self) -> List[G1Element]:
         """
         Returns all public keys.
         """
-        all_keys: List[Tuple[G1Element, bytes]] = []
-
-        index = 0
-        pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
-        while index <= MAX_KEYS:
-            if pkent is not None:
-                pk, ent = pkent
-                all_keys.append(pk)
-            index += 1
-            pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
-        return all_keys
+        return [key_data[0].get_g1() for key_data in self.get_all_private_keys()]
 
     def get_first_public_key(self) -> Optional[G1Element]:
         """
         Returns the first public key.
         """
-        index = 0
-        pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
-        while index <= MAX_KEYS:
-            if pkent is not None:
-                pk, ent = pkent
-                return pk
-            index += 1
-            pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
-        return None
+        key_data = self.get_first_private_key()
+        return None if key_data is None else key_data[0].get_g1()
 
-    def delete_key_by_fingerprint(self, fingerprint: int):
+    def delete_key_by_fingerprint(self, fingerprint: int) -> int:
         """
-        Deletes all keys which have the given public key fingerprint.
+        Deletes all keys which have the given public key fingerprint and returns how many keys were removed.
         """
-
-        index = 0
-        pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
-        while index <= MAX_KEYS:
-            if pkent is not None:
-                pk, ent = pkent
-                if pk.get_fingerprint() == fingerprint:
-                    self.keyring_wrapper.delete_passphrase(self.service, get_private_key_user(self.user, index))
-            index += 1
-            pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
+        removed = 0
+        for index in range(MAX_KEYS + 1):
+            try:
+                key_data = self._get_key_data(index, include_secrets=False)
+                if key_data.fingerprint == fingerprint:
+                    try:
+                        self.keyring_wrapper.delete_label(key_data.fingerprint)
+                    except (KeychainException, NotImplementedError):
+                        # Just try to delete the label and move on if there wasn't one
+                        pass
+                    try:
+                        self.keyring_wrapper.delete_passphrase(self.service, get_private_key_user(self.user, index))
+                        removed += 1
+                    except Exception:
+                        pass
+            except KeychainUserNotFound:
+                pass
+        return removed
 
     def delete_keys(self, keys_to_delete: List[Tuple[PrivateKey, bytes]]):
         """
         Deletes all keys in the list.
         """
-        remaining_keys = {str(x[0]) for x in keys_to_delete}
-        index = 0
-        pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
-        while index <= MAX_KEYS and len(remaining_keys) > 0:
-            if pkent is not None:
-                mnemonic = bytes_to_mnemonic(pkent[1])
-                seed = mnemonic_to_seed(mnemonic, "")
-                sk = AugSchemeMPL.key_gen(seed)
-                sk_str = str(sk)
-                if sk_str in remaining_keys:
-                    self.keyring_wrapper.delete_passphrase(self.service, get_private_key_user(self.user, index))
-                    remaining_keys.remove(sk_str)
-            index += 1
-            pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
-        if len(remaining_keys) > 0:
-            raise ValueError(f"{len(remaining_keys)} keys could not be found for deletion")
+        remaining_fingerprints = {x[0].get_g1().get_fingerprint() for x in keys_to_delete}
+        remaining_removals = len(remaining_fingerprints)
+        while len(remaining_fingerprints):
+            key_to_delete = remaining_fingerprints.pop()
+            if self.delete_key_by_fingerprint(key_to_delete) > 0:
+                remaining_removals -= 1
+        if remaining_removals > 0:
+            raise ValueError(f"{remaining_removals} keys could not be found for deletion")
 
     def delete_all_keys(self) -> None:
         """
         Deletes all keys from the keychain.
         """
-
-        index = 0
-        delete_exception = False
-        pkent = None
-        while True:
+        for index in range(MAX_KEYS + 1):
             try:
-                pkent = self._get_pk_and_entropy(get_private_key_user(self.user, index))
-                self.keyring_wrapper.delete_passphrase(self.service, get_private_key_user(self.user, index))
-            except Exception:
-                # Some platforms might throw on no existing key
-                delete_exception = True
-
-            # Stop when there are no more keys to delete
-            if (pkent is None or delete_exception) and index > MAX_KEYS:
-                break
-            index += 1
-
-        index = 0
-        delete_exception = True
-        pkent = None
-        while True:
-            try:
-                pkent = self._get_pk_and_entropy(
-                    get_private_key_user(self.user, index)
-                )  # changed from _get_fingerprint_and_entropy to _get_pk_and_entropy - GH
-                self.keyring_wrapper.delete_passphrase(self.service, get_private_key_user(self.user, index))
-            except Exception:
-                # Some platforms might throw on no existing key
-                delete_exception = True
-
-            # Stop when there are no more keys to delete
-            if (pkent is None or delete_exception) and index > MAX_KEYS:
-                break
-            index += 1
+                key_data = self._get_key_data(index)
+                self.delete_key_by_fingerprint(key_data.fingerprint)
+            except KeychainUserNotFound:
+                pass
 
     @staticmethod
     def is_keyring_locked() -> bool:
