@@ -1,4 +1,4 @@
-from blspy import PrivateKey  # pyright: reportMissingImports=false
+from chinillablspy import PrivateKey  # pyright: reportMissingImports=false
 from chinilla.util.default_root import DEFAULT_KEYS_ROOT_PATH
 from chinilla.util.file_keyring import FileKeyring
 from chinilla.util.misc import prompt_yes_no
@@ -113,11 +113,11 @@ class KeyringWrapper:
         if force_legacy:
             legacy_keyring = get_legacy_keyring_instance()
             if check_legacy_keyring_keys_present(legacy_keyring):
-                self.keyring = legacy_keyring
+                self.legacy_keyring = legacy_keyring
         else:
             self.refresh_keyrings()
 
-        if self.keyring is None:
+        if self.keyring is None and self.legacy_keyring is None:
             raise KeychainNotSet(
                 f"Unable to initialize keyring backend: keys_root_path={keys_root_path}, force_legacy={force_legacy}"
             )
@@ -284,7 +284,7 @@ class KeyringWrapper:
                 passphrase_store.set_password(MASTER_PASSPHRASE_SERVICE_NAME, MASTER_PASSPHRASE_USER_NAME, passphrase)
             except KeyringError as e:
                 if not warn_if_macos_errSecInteractionNotAllowed(e):
-                    raise e
+                    raise
         return None
 
     def remove_master_passphrase_from_credential_store(self) -> None:
@@ -292,15 +292,15 @@ class KeyringWrapper:
         if passphrase_store is not None:
             try:
                 passphrase_store.delete_password(MASTER_PASSPHRASE_SERVICE_NAME, MASTER_PASSPHRASE_USER_NAME)
-            except PasswordDeleteError as e:
+            except PasswordDeleteError:
                 if (
                     passphrase_store.get_credential(MASTER_PASSPHRASE_SERVICE_NAME, MASTER_PASSPHRASE_USER_NAME)
                     is not None
                 ):
-                    raise e
+                    raise
             except KeyringError as e:
                 if not warn_if_macos_errSecInteractionNotAllowed(e):
-                    raise e
+                    raise
         return None
 
     def get_master_passphrase_from_credential_store(self) -> Optional[str]:
@@ -310,7 +310,7 @@ class KeyringWrapper:
                 return passphrase_store.get_password(MASTER_PASSPHRASE_SERVICE_NAME, MASTER_PASSPHRASE_USER_NAME)
             except KeyringError as e:
                 if not warn_if_macos_errSecInteractionNotAllowed(e):
-                    raise e
+                    raise
         return None
 
     def get_master_passphrase_hint(self) -> Optional[str]:
@@ -349,8 +349,8 @@ class KeyringWrapper:
                 "passphrase."
             )
             print(
-                "Would you like to set a master passphrase now? Use 'chinilla passphrase set' to change the "
-                "passphrase.\n"
+                "Would you like to set a master passphrase now? Use 'chinilla passphrase set' to change the"
+                " passphrase.\n"
             )
 
             response = prompt_yes_no("Set keyring master passphrase?")
@@ -367,8 +367,8 @@ class KeyringWrapper:
                 )
             else:
                 print(
-                    "Will skip setting a master passphrase. Use 'chinilla passphrase set' to set the master "
-                    "passphrase.\n"
+                    "Will skip setting a master passphrase. Use 'chinilla passphrase set' to set the master"
+                    " passphrase.\n"
                 )
         else:
             import colorama
@@ -445,7 +445,7 @@ class KeyringWrapper:
             print(f"\nMigration failed: {e}")
             print("Leaving legacy keyring intact")
             self.legacy_keyring = migration_results.legacy_keyring  # Restore the legacy keyring
-            raise e
+            raise
 
         return success
 
@@ -527,7 +527,8 @@ class KeyringWrapper:
         # Continue reading from the legacy keyring until we want to write something,
         # at which point we'll migrate the legacy contents to the new keyring
         if self.using_legacy_keyring():
-            return self.legacy_keyring.get_password(service, user)  # type: ignore
+            passphrase = self.legacy_keyring.get_password(service, user)  # type: ignore
+            return passphrase.hex() if type(passphrase) == bytes else passphrase
 
         return self.get_keyring().get_password(service, user)
 
@@ -536,3 +537,21 @@ class KeyringWrapper:
 
     def delete_passphrase(self, service: str, user: str):
         self.get_keyring().delete_password(service, user)
+
+    def get_label(self, fingerprint: int) -> Optional[str]:
+        if self.using_legacy_keyring():
+            return None  # Legacy keyring doesn't support key labels
+
+        return self.keyring.get_label(fingerprint)
+
+    def set_label(self, fingerprint: int, label: str) -> None:
+        if self.using_legacy_keyring():
+            raise NotImplementedError("Legacy keyring doesn't support key labels")
+
+        self.keyring.set_label(fingerprint, label)
+
+    def delete_label(self, fingerprint: int) -> None:
+        if self.using_legacy_keyring():
+            raise NotImplementedError("Legacy keyring doesn't support key labels")
+
+        self.keyring.delete_label(fingerprint)
