@@ -1,14 +1,72 @@
+from __future__ import annotations
+
+import asyncio
+import logging
 import socket
-from pathlib import Path
+import ssl
+
+from aiohttp import web
+from aiohttp.log import web_logger
+from dataclasses import dataclass
 from ipaddress import ip_address, IPv4Network, IPv6Network
 from typing import Iterable, List, Tuple, Union, Any, Optional, Dict
+from typing_extensions import final
 from chinilla.server.outbound_message import NodeType
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.peer_info import PeerInfo
-from chinilla.util.config import load_config
 from chinilla.util.ints import uint16
 
 
+@final
+@dataclass
+class WebServer:
+    runner: web.AppRunner
+    listen_port: uint16
+    _close_task: Optional[asyncio.Task[None]] = None
+
+    @classmethod
+    async def create(
+        cls,
+        hostname: str,
+        port: uint16,
+        routes: List[web.RouteDef],
+        max_request_body_size: int = 1024**2,  # Default `client_max_size` from web.Application
+        ssl_context: Optional[ssl.SSLContext] = None,
+        keepalive_timeout: int = 75,  # Default from aiohttp.web
+        shutdown_timeout: int = 60,  # Default `shutdown_timeout` from web.TCPSite
+        prefer_ipv6: bool = False,
+        logger: logging.Logger = web_logger,
+    ) -> WebServer:
+        app = web.Application(client_max_size=max_request_body_size, logger=logger)
+        runner = web.AppRunner(app, access_log=None, keepalive_timeout=keepalive_timeout)
+
+        runner.app.add_routes(routes)
+        await runner.setup()
+        site = web.TCPSite(runner, hostname, int(port), ssl_context=ssl_context, shutdown_timeout=shutdown_timeout)
+        await site.start()
+
+        #
+        # On a dual-stack system, we want to get the (first) IPv4 port unless
+        # prefer_ipv6 is set in which case we use the IPv6 port
+        #
+        if port == 0:
+            port = select_port(prefer_ipv6, runner.addresses)
+
+        return cls(runner=runner, listen_port=uint16(port))
+
+    async def _close(self) -> None:
+        await self.runner.shutdown()
+        await self.runner.cleanup()
+
+    def close(self) -> None:
+        self._close_task = asyncio.create_task(self._close())
+
+    async def await_closed(self) -> None:
+        if self._close_task is None:
+            raise RuntimeError("WebServer stop not triggered")
+        await self._close_task
+
+
 def is_in_network(peer_host: str, networks: Iterable[Union[IPv4Network, IPv6Network]]) -> bool:
     try:
         peer_host_ip = ip_address(peer_host)
@@ -52,7 +110,7 @@ def class_for_type(type: NodeType) -> Any:
 def get_host_addr(host: Union[PeerInfo, str], prefer_ipv6: Optional[bool]) -> str:
     # If there was no preference passed in (from config), set the system-wise
     # default here.  Not a great place to locate a default value, and we should
-    # probabaly do something to write it into the config, but.  For now...
+    # probably do something to write it into the config, but.  For now...
     if prefer_ipv6 is None:
         prefer_ipv6 = False
     # Use PeerInfo.is_valid() to see if it's already an address
@@ -88,9 +146,7 @@ def is_trusted_inner(peer_host: str, peer_node_id: bytes32, trusted_peers: Dict,
     return True
 
 
-def select_port(root_path: Path, addresses: List[Any]) -> uint16:
-    global_config = load_config(root_path, "config.yaml")
-    prefer_ipv6 = global_config.get("prefer_ipv6", False)
+def select_port(prefer_ipv6: bool, addresses: List[Any]) -> uint16:
     selected_port: uint16
     for address_string, port, *_ in addresses:
         address = ip_address(address_string)
