@@ -4,6 +4,7 @@ import dataclasses
 import io
 import os
 import pprint
+import traceback
 from enum import Enum
 from typing import (
     Any,
@@ -21,7 +22,7 @@ from typing import (
     get_type_hints,
 )
 
-from blspy import G1Element, G2Element, PrivateKey
+from chinillablspy import G1Element, G2Element, PrivateKey
 from typing_extensions import Literal, get_args, get_origin
 
 from chinilla.types.blockchain_format.sized_bytes import bytes32
@@ -36,6 +37,10 @@ class StreamableError(Exception):
     pass
 
 
+class UnsupportedType(StreamableError):
+    pass
+
+
 class DefinitionError(StreamableError):
     def __init__(self, message: str, cls: Type[object]):
         super().__init__(
@@ -44,6 +49,35 @@ class DefinitionError(StreamableError):
         )
 
 
+class ParameterMissingError(StreamableError):
+    def __init__(self, cls: type, missing: List[str]):
+        super().__init__(
+            f"{len(missing)} field{'s' if len(missing) != 1 else ''} missing for {cls.__name__}: {', '.join(missing)}"
+        )
+
+
+class InvalidTypeError(StreamableError):
+    def __init__(self, expected: type, actual: type):
+        super().__init__(
+            f"Invalid type: Expected {expected.__name__}, Actual: {actual.__name__}",
+        )
+
+
+class InvalidSizeError(StreamableError):
+    def __init__(self, expected: int, actual: int):
+        super().__init__(
+            f"Invalid size: Expected {expected}, Actual: {actual}",
+        )
+
+
+class ConversionError(StreamableError):
+    def __init__(self, value: object, to_type: type, exception: Exception):
+        super().__init__(
+            f"Failed to convert {value!r} from type {type(value).__name__} to {to_type.__name__}: "
+            + "".join(traceback.format_exception_only(type(exception), value=exception)).strip()
+        )
+
+
 # TODO: Remove hack, this allows streaming these objects from binary
 size_hints = {
     "PrivateKey": PrivateKey.PRIVATE_KEY_SIZE,
@@ -123,26 +157,26 @@ def convert_optional(convert_func: ConvertFunctionType, item: Any) -> Any:
 
 
 def convert_tuple(convert_funcs: List[ConvertFunctionType], items: Collection[Any]) -> Tuple[Any, ...]:
-    if len(items) != len(convert_funcs):
-        raise ValueError(f"Invalid size. Expected: {len(convert_funcs)}, got: {len(items)}")
     if not isinstance(items, (list, tuple)):
-        raise TypeError(f"expected: tuple or list, actual: {type(items).__name__}")
+        raise InvalidTypeError(tuple, type(items))
+    if len(items) != len(convert_funcs):
+        raise InvalidSizeError(len(convert_funcs), len(items))
     return tuple(convert_func(item) for convert_func, item in zip(convert_funcs, items))
 
 
 def convert_list(convert_func: ConvertFunctionType, items: List[Any]) -> List[Any]:
     if not isinstance(items, list):
-        raise TypeError(f"expected: list, actual: {type(items).__name__}")
+        raise InvalidTypeError(list, type(items))
     return [convert_func(item) for item in items]
 
 
 def convert_hex_string(item: str) -> bytes:
     if not isinstance(item, str):
-        raise TypeError(f"expected: hex-string, actual: {type(item).__name__}")
+        raise InvalidTypeError(str, type(item))
     try:
         return hexstr_to_bytes(item)
     except Exception as e:
-        raise TypeError(f"Can't convert the string {item!r} to bytes: {e}") from e
+        raise ConversionError(item, bytes, e) from e
 
 
 def convert_byte_type(f_type: Type[Any], item: Any) -> Any:
@@ -153,7 +187,7 @@ def convert_byte_type(f_type: Type[Any], item: Any) -> Any:
     try:
         return f_type(item)
     except Exception as e:
-        raise TypeError(f"Can't convert {type(item).__name__} to {f_type.__name__}: {e}") from e
+        raise ConversionError(item, f_type, e) from e
 
 
 def convert_unhashable_type(f_type: Type[Any], item: Any) -> Any:
@@ -167,7 +201,7 @@ def convert_unhashable_type(f_type: Type[Any], item: Any) -> Any:
         else:
             return f_type.from_bytes(item)
     except Exception as e:
-        raise TypeError(f"Can't convert {type(item).__name__} to {f_type.__name__}: {e}") from e
+        raise ConversionError(item, f_type, e) from e
 
 
 def convert_primitive(f_type: Type[Any], item: Any) -> Any:
@@ -176,7 +210,7 @@ def convert_primitive(f_type: Type[Any], item: Any) -> Any:
     try:
         return f_type(item)
     except Exception as e:
-        raise TypeError(f"Can't convert type {type(item).__name__} to {f_type.__name__}: {e}") from e
+        raise ConversionError(item, f_type, e) from e
 
 
 def streamable_from_dict(klass: Type[_T_Streamable], item: Any) -> _T_Streamable:
@@ -187,7 +221,7 @@ def streamable_from_dict(klass: Type[_T_Streamable], item: Any) -> _T_Streamable
     if isinstance(item, klass):
         return item
     if not isinstance(item, dict):
-        raise TypeError(f"expected: dict, actual: {type(item).__name__}")
+        raise InvalidTypeError(dict, type(item))
 
     fields = klass.streamable_fields()
     try:
@@ -195,10 +229,7 @@ def streamable_from_dict(klass: Type[_T_Streamable], item: Any) -> _T_Streamable
     except TypeError as e:
         missing_fields = [field.name for field in fields if field.name not in item and not field.has_default]
         if len(missing_fields) > 0:
-            raise KeyError(
-                f"{len(missing_fields)} field{'s' if len(missing_fields) > 1 else ''} missing for {klass.__name__}: "
-                + ", ".join(missing_fields)
-            ) from e
+            raise ParameterMissingError(klass, missing_fields) from e
         raise
 
 
@@ -245,7 +276,7 @@ def post_init_process_item(f_type: Type[Any], item: Any) -> object:
             except Exception:
                 item = from_bytes_method(bytes(item))
     if not isinstance(item, f_type):
-        raise ValueError(f"Wrong type for {f_type}")
+        raise InvalidTypeError(f_type, type(item))
     return item
 
 
@@ -302,7 +333,7 @@ def recurse_jsonify(d: Any) -> Any:
     elif hasattr(d, "to_json_dict"):
         ret: Union[List[Any], Dict[str, Any], str, None, int] = d.to_json_dict()
         return ret
-    raise ValueError(f"failed to jsonify {d} (type: {type(d)})")
+    raise UnsupportedType(f"failed to jsonify {d} (type: {type(d)})")
 
 
 def parse_bool(f: BinaryIO) -> bool:
@@ -340,7 +371,7 @@ def parse_optional(f: BinaryIO, parse_inner_type_f: ParseFunctionType) -> Option
 def parse_rust(f: BinaryIO, f_type: Type[Any]) -> Any:
     assert isinstance(f, io.BytesIO)
     buf = f.getbuffer()
-    ret, advance = f_type.parse_rust(bytes(buf[f.tell() :]))
+    ret, advance = f_type.parse_rust(buf[f.tell() :])
     f.seek(advance, os.SEEK_CUR)
     return ret
 
@@ -419,7 +450,7 @@ def function_to_parse_one_item(f_type: Type[Any]) -> ParseFunctionType:
         return lambda f: parse_size_hints(f, f_type, bytes_to_read, unchecked=False)
     if f_type is str:
         return parse_str
-    raise NotImplementedError(f"Type {f_type} does not have parse")
+    raise UnsupportedType(f"Type {f_type} does not have parse")
 
 
 def stream_optional(stream_inner_type_func: StreamFunctionType, item: Any, f: BinaryIO) -> None:
@@ -492,7 +523,7 @@ def function_to_stream_one_item(f_type: Type[Any]) -> StreamFunctionType:
     elif f_type is bool:
         return stream_bool
     else:
-        raise NotImplementedError(f"can't stream {f_type}")
+        raise UnsupportedType(f"can't stream {f_type}")
 
 
 def streamable(cls: Type[_T_Streamable]) -> Type[_T_Streamable]:
@@ -584,10 +615,14 @@ class Streamable:
 
     def __post_init__(self) -> None:
         data = self.__dict__
-        for field in self._streamable_fields:
-            if field.name not in data:
-                raise ValueError(f"Field {field.name} not present")
-            object.__setattr__(self, field.name, field.post_init_function(data[field.name]))
+        try:
+            for field in self._streamable_fields:
+                object.__setattr__(self, field.name, field.post_init_function(data[field.name]))
+        except TypeError as e:
+            missing_fields = [field.name for field in self._streamable_fields if field.name not in data]
+            if len(missing_fields) > 0:
+                raise ParameterMissingError(type(self), missing_fields) from e
+            raise
 
     @classmethod
     def parse(cls: Type[_T_Streamable], f: BinaryIO) -> _T_Streamable:
@@ -605,7 +640,7 @@ class Streamable:
         return std_hash(bytes(self), skip_bytes_conversion=True)
 
     @classmethod
-    def from_bytes(cls: Any, blob: bytes) -> Any:
+    def from_bytes(cls: Type[_T_Streamable], blob: bytes) -> _T_Streamable:
         f = io.BytesIO(blob)
         parsed = cls.parse(f)
         assert f.read() == b""
