@@ -12,12 +12,14 @@ you can use p2_conditions.
 This roughly corresponds to bitcoin's graftroot.
 """
 
+from __future__ import annotations
+
 from chinilla.types.blockchain_format.program import Program
 
 from . import p2_conditions
-from .load_clvm import load_clvm
+from .load_clvm import load_clvm_maybe_recompile
 
-MOD = load_clvm("p2_delegated_puzzle.clvm")
+MOD = load_clvm_maybe_recompile("p2_delegated_puzzle.clvm")
 
 
 def puzzle_for_pk(public_key: bytes) -> Program:
