@@ -1,7 +1,8 @@
+from __future__ import annotations
 from dataclasses import dataclass
-from typing import Any, Dict, List, Optional, Set, Tuple
+from typing import Any, Dict, List, Optional, Set, Tuple, Union, BinaryIO
 
-from blspy import G2Element
+from chinillablspy import G2Element
 from clvm_tools.binutils import disassemble
 
 from chinilla.types.blockchain_format.sized_bytes import bytes32
@@ -22,14 +23,18 @@ from chinilla.wallet.outer_puzzles import (
 )
 from chinilla.wallet.payment import Payment
 from chinilla.wallet.puzzle_drivers import PuzzleInfo, Solver
-from chinilla.wallet.puzzles.load_clvm import load_clvm
+from chinilla.wallet.puzzles.load_clvm import load_clvm_maybe_recompile
 from chinilla.wallet.util.puzzle_compression import (
     compress_object_with_puzzles,
     decompress_object_with_puzzles,
     lowest_best_version,
 )
+from chinilla.wallet.uncurried_puzzle import UncurriedPuzzle, uncurry_puzzle
 
-OFFER_MOD = load_clvm("settlement_payments.clvm")
+OFFER_MOD_OLD = load_clvm_maybe_recompile("settlement_payments_old.clvm")
+OFFER_MOD = load_clvm_maybe_recompile("settlement_payments_old.clvm")
+OFFER_MOD_OLD_HASH = OFFER_MOD_OLD.get_tree_hash()
+OFFER_MOD_HASH = OFFER_MOD.get_tree_hash()
 ZERO_32 = bytes32([0] * 32)
 
 
@@ -67,8 +72,8 @@ class Offer:
     driver_dict: Dict[bytes32, PuzzleInfo]  # asset_id -> asset driver
 
     @staticmethod
-    def ph():
-        return OFFER_MOD.get_tree_hash()
+    def ph() -> bytes32:
+        return OFFER_MOD_HASH
 
     @staticmethod
     def notarize_payments(
@@ -77,7 +82,7 @@ class Offer:
     ) -> Dict[Optional[bytes32], List[NotarizedPayment]]:
         # This sort should be reproducible in CLVM with `>s`
         sorted_coins: List[Coin] = sorted(coins, key=Coin.name)
-        sorted_coin_list: List[List] = [coin_as_list(c) for c in sorted_coins]
+        sorted_coin_list: List[List[Union[bytes32, uint64]]] = [coin_as_list(c) for c in sorted_coins]
         nonce: bytes32 = Program.to(sorted_coin_list).get_tree_hash()
 
         notarized_payments: Dict[Optional[bytes32], List[NotarizedPayment]] = {}
@@ -101,14 +106,14 @@ class Offer:
                     raise ValueError("Cannot calculate announcements without driver of requested item")
                 settlement_ph: bytes32 = construct_puzzle(driver_dict[asset_id], OFFER_MOD).get_tree_hash()
             else:
-                settlement_ph = OFFER_MOD.get_tree_hash()
+                settlement_ph = OFFER_MOD_HASH
 
             msg: bytes32 = Program.to((payments[0].nonce, [p.as_condition_args() for p in payments])).get_tree_hash()
             announcements.append(Announcement(settlement_ph, msg))
 
         return announcements
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         # Verify that there are no duplicate payments
         for payments in self.requested_payments.values():
             payment_programs: List[bytes32] = [p.name() for p in payments]
@@ -146,11 +151,10 @@ class Offer:
     def get_offered_coins(self) -> Dict[Optional[bytes32], List[Coin]]:
         offered_coins: Dict[Optional[bytes32], List[Coin]] = {}
 
-        OFFER_HASH: bytes32 = OFFER_MOD.get_tree_hash()
         for parent_spend in self.bundle.coin_spends:
             coins_for_this_spend: List[Coin] = []
 
-            parent_puzzle: Program = parent_spend.puzzle_reveal.to_program()
+            parent_puzzle: UncurriedPuzzle = uncurry_puzzle(parent_spend.puzzle_reveal.to_program())
             parent_solution: Program = parent_spend.solution.to_program()
             additions: List[Coin] = [a for a in parent_spend.additions() if a not in self.bundle.removals()]
 
@@ -163,7 +167,7 @@ class Offer:
                 conditions: Program = inner_puzzle.run(inner_solution)
                 matching_spend_additions: List[Coin] = []  # coins that match offered amount and are sent to offer ph.
                 for condition in conditions.as_iter():
-                    if condition.first() == 51 and condition.rest().first() == OFFER_HASH:
+                    if condition.first() == 51 and condition.rest().first() in [OFFER_MOD_HASH, OFFER_MOD_OLD_HASH]:
                         matching_spend_additions.extend(
                             [a for a in additions if a.amount == condition.rest().rest().first().as_int()]
                         )
@@ -174,13 +178,22 @@ class Offer:
                         a
                         for a in matching_spend_additions
                         if a.puzzle_hash
-                        == construct_puzzle(puzzle_driver, OFFER_HASH).get_tree_hash(OFFER_HASH)  # type: ignore
+                        in [
+                            construct_puzzle(puzzle_driver, OFFER_MOD_OLD_HASH).get_tree_hash_precalc(  # type: ignore
+                                OFFER_MOD_OLD_HASH
+                            ),
+                            construct_puzzle(puzzle_driver, OFFER_MOD_HASH).get_tree_hash_precalc(  # type: ignore
+                                OFFER_MOD_HASH
+                            ),
+                        ]
                     ]
                     if len(additions_w_amount_and_puzhash) == 1:
                         coins_for_this_spend.append(additions_w_amount_and_puzhash[0])
             else:
                 asset_id = None
-                coins_for_this_spend.extend([a for a in additions if a.puzzle_hash == OFFER_HASH])
+                coins_for_this_spend.extend(
+                    [a for a in additions if a.puzzle_hash in [OFFER_MOD_HASH, OFFER_MOD_OLD_HASH]]
+                )
 
             if coins_for_this_spend != []:
                 offered_coins.setdefault(asset_id, [])
@@ -340,7 +353,7 @@ class Offer:
         return [cs.coin for cs in self.bundle.coin_spends if cs.coin.name() in coin_names]
 
     @classmethod
-    def aggregate(cls, offers: List["Offer"]) -> "Offer":
+    def aggregate(cls, offers: List[Offer]) -> Offer:
         total_requested_payments: Dict[Optional[bytes32], List[NotarizedPayment]] = {}
         total_bundle = SpendBundle([], G2Element())
         total_driver_dict: Dict[bytes32, PuzzleInfo] = {}
@@ -410,11 +423,17 @@ class Offer:
 
             for coin in offered_coins:
                 if asset_id:
+                    if coin.puzzle_hash == construct_puzzle(
+                        self.driver_dict[asset_id], OFFER_MOD_OLD_HASH  # type: ignore
+                    ).get_tree_hash_precalc(OFFER_MOD_OLD_HASH):
+                        offer_mod: Program = OFFER_MOD_OLD
+                    else:
+                        offer_mod = OFFER_MOD
                     siblings: str = "("
                     sibling_spends: str = "("
                     sibling_puzzles: str = "("
                     sibling_solutions: str = "("
-                    disassembled_offer_mod: str = disassemble(OFFER_MOD)
+                    disassembled_offer_mod: str = disassemble(offer_mod)  # type: ignore
                     for sibling_coin in offered_coins:
                         if sibling_coin != coin:
                             siblings += (
@@ -422,10 +441,11 @@ class Offer:
                                 + sibling_coin.parent_coin_info.hex()
                                 + sibling_coin.puzzle_hash.hex()
                                 + bytes(uint64(sibling_coin.amount)).hex()
+                                + " "
                             )
-                            sibling_spends += "0x" + bytes(coin_to_spend_dict[sibling_coin]).hex() + ")"
-                            sibling_puzzles += disassembled_offer_mod
-                            sibling_solutions += disassemble(coin_to_solution_dict[sibling_coin])
+                            sibling_spends += "0x" + bytes(coin_to_spend_dict[sibling_coin]).hex() + " "
+                            sibling_puzzles += disassembled_offer_mod + " "
+                            sibling_solutions += disassemble(coin_to_solution_dict[sibling_coin]) + " "  # type: ignore
                     siblings += ")"
                     sibling_spends += ")"
                     sibling_puzzles += ")"
@@ -446,16 +466,20 @@ class Offer:
                                 "sibling_solutions": sibling_solutions,
                             }
                         ),
-                        OFFER_MOD,
+                        offer_mod,
                         Program.to(coin_to_solution_dict[coin]),
                     )
                 else:
+                    if coin.puzzle_hash == OFFER_MOD_OLD_HASH:
+                        offer_mod = OFFER_MOD_OLD
+                    else:
+                        offer_mod = OFFER_MOD
                     solution = Program.to(coin_to_solution_dict[coin])
 
                 completion_spends.append(
                     CoinSpend(
                         coin,
-                        construct_puzzle(self.driver_dict[asset_id], OFFER_MOD) if asset_id else OFFER_MOD,
+                        construct_puzzle(self.driver_dict[asset_id], offer_mod) if asset_id else offer_mod,
                         solution,
                     )
                 )
@@ -493,13 +517,13 @@ class Offer:
         )
 
     @classmethod
-    def from_spend_bundle(cls, bundle: SpendBundle) -> "Offer":
+    def from_spend_bundle(cls, bundle: SpendBundle) -> Offer:
         # Because of the `to_spend_bundle` method, we need to parse the dummy CoinSpends as `requested_payments`
         requested_payments: Dict[Optional[bytes32], List[NotarizedPayment]] = {}
         driver_dict: Dict[bytes32, PuzzleInfo] = {}
         leftover_coin_spends: List[CoinSpend] = []
         for coin_spend in bundle.coin_spends:
-            driver = match_puzzle(coin_spend.puzzle_reveal.to_program())
+            driver = match_puzzle(uncurry_puzzle(coin_spend.puzzle_reveal.to_program()))
             if driver is not None:
                 asset_id = create_asset_id(driver)
                 assert asset_id is not None
@@ -524,32 +548,32 @@ class Offer:
     def name(self) -> bytes32:
         return self.to_spend_bundle().name()
 
-    def compress(self, version=None) -> bytes:
+    def compress(self, version: Optional[int] = None) -> bytes:
         as_spend_bundle = self.to_spend_bundle()
         if version is None:
             mods: List[bytes] = [bytes(s.puzzle_reveal.to_program().uncurry()[0]) for s in as_spend_bundle.coin_spends]
-            version = max(lowest_best_version(mods), 2)  # 2 is the version where OFFER_MOD lives
+            version = max(lowest_best_version(mods), 5)  # 5 is the version where OFFER_MOD lives
         return compress_object_with_puzzles(bytes(as_spend_bundle), version)
 
     @classmethod
-    def from_compressed(cls, compressed_bytes: bytes) -> "Offer":
+    def from_compressed(cls, compressed_bytes: bytes) -> Offer:
         return Offer.from_bytes(decompress_object_with_puzzles(compressed_bytes))
 
     @classmethod
-    def try_offer_decompression(cls, offer_bytes: bytes) -> "Offer":
+    def try_offer_decompression(cls, offer_bytes: bytes) -> Offer:
         try:
             return cls.from_compressed(offer_bytes)
         except TypeError:
             pass
         return cls.from_bytes(offer_bytes)
 
-    def to_bech32(self, prefix: str = "offer", compression_version=None) -> str:
+    def to_bech32(self, prefix: str = "offer", compression_version: Optional[int] = None) -> str:
         offer_bytes = self.compress(version=compression_version)
         encoded = bech32_encode(prefix, convertbits(list(offer_bytes), 8, 5))
         return encoded
 
     @classmethod
-    def from_bech32(cls, offer_bech32: str) -> "Offer":
+    def from_bech32(cls, offer_bech32: str) -> Offer:
         hrpgot, data = bech32_decode(offer_bech32, max_length=len(offer_bech32))
         if data is None:
             raise ValueError("Invalid Offer")
@@ -560,11 +584,11 @@ class Offer:
     # Methods to make this a valid Streamable member
     # We basically hijack the SpendBundle versions for most of it
     @classmethod
-    def parse(cls, f) -> "Offer":
+    def parse(cls, f: BinaryIO) -> Offer:
         parsed_bundle = SpendBundle.parse(f)
         return cls.from_bytes(bytes(parsed_bundle))
 
-    def stream(self, f):
+    def stream(self, f: BinaryIO) -> None:
         as_spend_bundle = SpendBundle.from_bytes(bytes(self))
         as_spend_bundle.stream(f)
 
@@ -572,7 +596,7 @@ class Offer:
         return bytes(self.to_spend_bundle())
 
     @classmethod
-    def from_bytes(cls, as_bytes: bytes) -> "Offer":
+    def from_bytes(cls, as_bytes: bytes) -> Offer:
         # Because of the __bytes__ method, we need to parse the dummy CoinSpends as `requested_payments`
         bundle = SpendBundle.from_bytes(as_bytes)
         return cls.from_spend_bundle(bundle)
