@@ -1,14 +1,15 @@
 import asyncio
 import dataclasses
-import json
 import logging
 import multiprocessing
 import random
+import sys
 import time
 import traceback
 from pathlib import Path
 from typing import Any, Callable, Dict, Iterator, List, Optional, Set, Tuple
-from blspy import AugSchemeMPL, PrivateKey, G2Element, G1Element
+
+from chinillablspy import AugSchemeMPL, PrivateKey, G2Element, G1Element
 from packaging.version import Version
 
 from chinilla.consensus.block_record import BlockRecord
@@ -28,6 +29,7 @@ from chinilla.protocols.wallet_protocol import (
     RespondToCoinUpdates,
     RespondToPhUpdates,
 )
+from chinilla.rpc.rpc_server import default_get_connections
 from chinilla.server.node_discovery import WalletPeers
 from chinilla.server.outbound_message import Message, NodeType, make_msg
 from chinilla.server.peer_store_resolver import PeerStoreResolver
@@ -41,7 +43,6 @@ from chinilla.types.header_block import HeaderBlock
 from chinilla.types.mempool_inclusion_status import MempoolInclusionStatus
 from chinilla.types.peer_info import PeerInfo
 from chinilla.types.weight_proof import WeightProof
-from chinilla.util.byte_types import hexstr_to_bytes
 from chinilla.util.chunks import chunks
 from chinilla.util.config import WALLET_PEERS_PATH_KEY_DEPRECATED, process_config_start_method
 from chinilla.util.errors import KeychainIsLocked, KeychainProxyConnectionFailure, KeychainIsEmpty, KeychainKeyNotFound
@@ -49,6 +50,7 @@ from chinilla.util.ints import uint32, uint64
 from chinilla.util.keychain import Keychain
 from chinilla.util.path import path_from_root
 from chinilla.util.profiler import profile_task
+from chinilla.util.memory_profiler import mem_profile_task
 from chinilla.wallet.transaction_record import TransactionRecord
 from chinilla.wallet.util.new_peak_queue import NewPeakItem, NewPeakQueue, NewPeakQueueTypes
 from chinilla.wallet.util.peer_request_cache import PeerRequestCache, can_use_peer_request_cache
@@ -56,13 +58,13 @@ from chinilla.wallet.util.wallet_sync_utils import (
     fetch_header_blocks_in_range,
     fetch_last_tx_from_peer,
     last_change_height_cs,
+    PeerRequestException,
     request_and_validate_additions,
     request_and_validate_removals,
     request_header_blocks,
     subscribe_to_coin_updates,
     subscribe_to_phs,
 )
-from chinilla.wallet.wallet_action import WalletAction
 from chinilla.wallet.wallet_state_manager import WalletStateManager
 from chinilla.wallet.wallet_weight_proof_handler import get_wp_fork_point, WalletWeightProofHandler
 
@@ -131,6 +133,7 @@ class WalletNode:
 
     _shut_down: bool = False
     _process_new_subscriptions_task: Optional[asyncio.Task] = None
+    _retry_failed_states_task: Optional[asyncio.Task] = None
     _primary_peer_sync_task: Optional[asyncio.Task] = None
     _secondary_peer_sync_task: Optional[asyncio.Task] = None
 
@@ -170,6 +173,9 @@ class WalletNode:
 
         return self._new_peak_queue
 
+    def get_connections(self, request_node_type: Optional[NodeType]) -> List[Dict[str, Any]]:
+        return default_get_connections(server=self.server, request_node_type=request_node_type)
+
     async def ensure_keychain_proxy(self) -> KeychainProxy:
         if self._keychain_proxy is None:
             if self.local_keychain:
@@ -207,7 +213,7 @@ class WalletNode:
         except KeychainProxyConnectionFailure as e:
             tb = traceback.format_exc()
             self.log.error(f"Missing keychain_proxy: {e} {tb}")
-            raise e  # Re-raise so that the caller can decide whether to continue or abort
+            raise  # Re-raise so that the caller can decide whether to continue or abort
 
         return key
 
@@ -226,7 +232,10 @@ class WalletNode:
 
         return key
 
-    async def _start(
+    async def _start(self) -> None:
+        await self._start_with_fingerprint()
+
+    async def _start_with_fingerprint(
         self,
         fingerprint: Optional[int] = None,
     ) -> bool:
@@ -245,7 +254,13 @@ class WalletNode:
             return False
 
         if self.config.get("enable_profiler", False):
-            asyncio.create_task(profile_task(self.root_path, "wallet", self.log))
+            if sys.getprofile() is not None:
+                self.log.warning("not enabling profiler, getprofile() is already set")
+            else:
+                asyncio.create_task(profile_task(self.root_path, "wallet", self.log))
+
+        if self.config.get("enable_memory_profiler", False):
+            asyncio.create_task(mem_profile_task(self.root_path, "wallet", self.log))
 
         path: Path = get_wallet_db_path(self.root_path, self.config, str(private_key.get_g1().get_fingerprint()))
         path.parent.mkdir(parents=True, exist_ok=True)
@@ -274,10 +289,12 @@ class WalletNode:
             self.wallet_state_manager.set_callback(self.state_changed_callback)
 
         self.last_wallet_tx_resend_time = int(time.time())
+        self.last_state_retry_time = int(time.time())
         self.wallet_tx_resend_timeout_secs = self.config.get("tx_resend_timeout_secs", 60 * 60)
         self.wallet_state_manager.set_pending_callback(self._pending_tx_handler)
         self._shut_down = False
         self._process_new_subscriptions_task = asyncio.create_task(self._process_new_subscriptions())
+        self._retry_failed_states_task = asyncio.create_task(self._retry_failed_states())
 
         self.sync_event = asyncio.Event()
         self.log_in(private_key)
@@ -297,6 +314,8 @@ class WalletNode:
 
         if self._process_new_subscriptions_task is not None:
             self._process_new_subscriptions_task.cancel()
+        if self._retry_failed_states_task is not None:
+            self._retry_failed_states_task.cancel()
         if self._primary_peer_sync_task is not None:
             self._primary_peer_sync_task.cancel()
         if self._secondary_peer_sync_task is not None:
@@ -332,25 +351,6 @@ class WalletNode:
             return None
         asyncio.create_task(self._resend_queue())
 
-    async def _action_messages(self) -> List[Message]:
-        if self._wallet_state_manager is None:
-            return []
-        actions: List[WalletAction] = await self.wallet_state_manager.action_store.get_all_pending_actions()
-        result: List[Message] = []
-        for action in actions:
-            data = json.loads(action.data)
-            action_data = data["data"]["action_data"]
-            if action.name == "request_puzzle_solution":
-                coin_name = bytes32(hexstr_to_bytes(action_data["coin_name"]))
-                height = uint32(action_data["height"])
-                msg = make_msg(
-                    ProtocolMessageTypes.request_puzzle_solution,
-                    wallet_protocol.RequestPuzzleSolution(coin_name, height),
-                )
-                result.append(msg)
-
-        return result
-
     async def _resend_queue(self):
         if self._shut_down or self._server is None or self._wallet_state_manager is None:
             return None
@@ -358,18 +358,13 @@ class WalletNode:
         for msg, sent_peers in await self._messages_to_resend():
             if self._shut_down or self._server is None or self._wallet_state_manager is None:
                 return None
-            full_nodes = self.server.get_full_node_connections()
+            full_nodes = self.server.get_connections(NodeType.FULL_NODE)
             for peer in full_nodes:
                 if peer.peer_node_id in sent_peers:
                     continue
                 self.log.debug(f"sending: {msg}")
                 await peer.send_message(msg)
 
-        for msg in await self._action_messages():
-            if self._shut_down or self._server is None or self._wallet_state_manager is None:
-                return None
-            await self.server.send_to_all([msg], NodeType.FULL_NODE)
-
     async def _messages_to_resend(self) -> List[Tuple[Message, Set[bytes32]]]:
         if self._wallet_state_manager is None or self._shut_down:
             return []
@@ -399,6 +394,45 @@ class WalletNode:
 
         return messages
 
+    async def _retry_failed_states(self):
+        while not self._shut_down:
+            try:
+                await asyncio.sleep(5)
+                current_time = time.time()
+                if self.last_state_retry_time < current_time - 10:
+                    self.last_state_retry_time = current_time
+                    if self.wallet_state_manager is None:
+                        continue
+                    states_to_retry = await self.wallet_state_manager.retry_store.get_all_states_to_retry()
+                    for state, peer_id, fork_height in states_to_retry:
+                        matching_peer = tuple(
+                            p for p in self.server.get_connections(NodeType.FULL_NODE) if p.peer_node_id == peer_id
+                        )
+                        if len(matching_peer) == 0:
+                            peer = self.get_full_node_peer()
+                            if peer is None:
+                                self.log.info(f"disconnected from all peers, cannot retry state: {state}")
+                                continue
+                            else:
+                                self.log.info(
+                                    f"disconnected from peer {peer_id}, state will retry with {peer.peer_node_id}"
+                                )
+                        else:
+                            peer = matching_peer[0]
+                        async with self.wallet_state_manager.db_wrapper.writer():
+                            self.log.info(f"retrying coin_state: {state}")
+                            try:
+                                await self.wallet_state_manager.new_coin_state(
+                                    [state], peer, None if fork_height == 0 else fork_height
+                                )
+                            except Exception as e:
+                                self.log.exception(f"Exception while adding states.. : {e}")
+                            else:
+                                await self.wallet_state_manager.blockchain.clean_block_records()
+            except asyncio.CancelledError:
+                self.log.info("Retry task cancelled, exiting.")
+                raise
+
     async def _process_new_subscriptions(self):
         while not self._shut_down:
             # Here we process four types of messages in the queue, where the first one has higher priority (lower
@@ -415,14 +449,14 @@ class WalletNode:
                     # state updates until we are sure that we subscribed to everything that we need to. Otherwise,
                     # we might not be able to process some state.
                     coin_ids: List[bytes32] = item.data
-                    for peer in self.server.get_full_node_connections():
+                    for peer in self.server.get_connections(NodeType.FULL_NODE):
                         coin_states: List[CoinState] = await subscribe_to_coin_updates(coin_ids, peer, uint32(0))
                         if len(coin_states) > 0:
                             async with self.wallet_state_manager.lock:
                                 await self.receive_state_from_peer(coin_states, peer)
                 elif item.item_type == NewPeakQueueTypes.PUZZLE_HASH_SUBSCRIPTION:
                     puzzle_hashes: List[bytes32] = item.data
-                    for peer in self.server.get_full_node_connections():
+                    for peer in self.server.get_connections(NodeType.FULL_NODE):
                         # Puzzle hash subscription
                         coin_states: List[CoinState] = await subscribe_to_phs(puzzle_hashes, peer, uint32(0))
                         if len(coin_states) > 0:
@@ -705,7 +739,7 @@ class WalletNode:
             self.validation_semaphore = asyncio.Semaphore(10)
 
         # Rollback is handled in wallet_short_sync_backtrack for untrusted peers, so we don't need to do it here.
-        # Also it's not safe to rollback, an untrusted peer can give us old fork point and make our TX dissapear.
+        # Also it's not safe to rollback, an untrusted peer can give us old fork point and make our TX disappear.
         # wallet_short_sync_backtrack can safely rollback because we validated the weight for the new peak so we
         # know the peer is telling the truth about the reorg.
 
@@ -733,9 +767,11 @@ class WalletNode:
 
         # Ensure the list is sorted
 
-        self.log.info(f"Filtering spam, total items: {len(items_input)}")
+        before = len(items_input)
         items = await self.wallet_state_manager.filter_spam(list(sorted(items_input, key=last_change_height_cs)))
-        self.log.info(f"Filtered spam. Total left: {len(items)}")
+        num_filtered = before - len(items)
+        if num_filtered > 0:
+            self.log.info(f"Filtered {num_filtered} spam transactions")
 
         async def receive_and_validate(inner_states: List[CoinState], inner_idx_start: int, cs_heights: List[uint32]):
             try:
@@ -776,7 +812,10 @@ class WalletNode:
 
             except Exception as e:
                 tb = traceback.format_exc()
-                self.log.error(f"Exception while adding state: {e} {tb}")
+                if self._shut_down:
+                    self.log.debug(f"Shutting down while adding state : {e} {tb}")
+                else:
+                    self.log.error(f"Exception while adding state: {e} {tb}")
             finally:
                 cs_heights.remove(last_change_height_cs(inner_states[0]))
 
@@ -827,10 +866,10 @@ class WalletNode:
 
     async def get_coins_with_puzzle_hash(self, puzzle_hash) -> List[CoinState]:
         # TODO Use trusted peer, otherwise try untrusted
-        all_nodes = self.server.connection_by_type[NodeType.FULL_NODE]
-        if len(all_nodes.keys()) == 0:
+        all_nodes = self.server.get_connections(NodeType.FULL_NODE)
+        if len(all_nodes) == 0:
             raise ValueError("Not connected to the full node")
-        first_node = list(all_nodes.values())[0]
+        first_node = all_nodes[0]
         msg = wallet_protocol.RegisterForPhUpdates(puzzle_hash, uint32(0))
         coin_state: Optional[RespondToPhUpdates] = await first_node.register_interest_in_puzzle_hash(msg)
         # TODO validate state if received from untrusted peer
@@ -907,7 +946,7 @@ class WalletNode:
         synced: List[WSChinillaConnection] = []
         trusted: List[WSChinillaConnection] = []
         neither: List[WSChinillaConnection] = []
-        all_nodes: List[WSChinillaConnection] = self.server.get_full_node_connections().copy()
+        all_nodes: List[WSChinillaConnection] = self.server.get_connections(NodeType.FULL_NODE)
         random.shuffle(all_nodes)
         for node in all_nodes:
             we_synced_to_it = node.peer_node_id in self.synced_peers
@@ -927,8 +966,9 @@ class WalletNode:
             return
 
         # Close connection of non-trusted peers
-        if len(self.server.get_full_node_connections()) > 1:
-            for peer in self.server.get_full_node_connections():
+        full_node_connections = self.server.get_connections(NodeType.FULL_NODE)
+        if len(full_node_connections) > 1:
+            for peer in full_node_connections:
                 if not self.is_trusted(peer):
                     await peer.close()
 
@@ -939,7 +979,7 @@ class WalletNode:
     async def check_for_synced_trusted_peer(self, header_block: HeaderBlock, request_time: uint64) -> bool:
         if self._server is None:
             return False
-        for peer in self.server.get_full_node_connections():
+        for peer in self.server.get_connections(NodeType.FULL_NODE):
             if self.is_trusted(peer) and await self.is_peer_synced(peer, header_block, request_time):
                 return True
         return False
@@ -968,7 +1008,7 @@ class WalletNode:
             self.get_cache_for_peer(peer).add_to_blocks(last_tx_block)
             return last_tx_block.foliage_transaction_block.timestamp
 
-        raise ValueError("Error fetching timestamp from all peers")
+        raise PeerRequestException("Error fetching timestamp from all peers")
 
     async def new_peak_wallet(self, new_peak: wallet_protocol.NewPeakWallet, peer: WSChinillaConnection):
         if self._wallet_state_manager is None:
@@ -1260,8 +1300,8 @@ class WalletNode:
 
     async def get_coin_ids_to_subscribe(self, min_height: int) -> List[bytes32]:
         all_coin_names: Set[bytes32] = await self.wallet_state_manager.coin_store.get_coin_names_to_check(min_height)
-        removed_dict = await self.wallet_state_manager.trade_manager.get_coins_of_interest()
-        all_coin_names.update(removed_dict.keys())
+        removed_names = await self.wallet_state_manager.trade_manager.get_coins_of_interest()
+        all_coin_names.update(set(removed_names))
         all_coin_names.update(await self.wallet_state_manager.interested_store.get_interested_coin_ids())
         return list(all_coin_names)
 
@@ -1281,8 +1321,10 @@ class WalletNode:
         if await can_use_peer_request_cache(coin_state, peer_request_cache, fork_height):
             return True
 
-        spent_height = coin_state.spent_height
-        confirmed_height = coin_state.created_height
+        spent_height: Optional[uint32] = None if coin_state.spent_height is None else uint32(coin_state.spent_height)
+        confirmed_height: Optional[uint32] = (
+            None if coin_state.created_height is None else uint32(coin_state.created_height)
+        )
         current = await self.wallet_state_manager.coin_store.get_coin_record(coin_state.coin.name())
         # if remote state is same as current local state we skip validation
 
@@ -1473,13 +1515,16 @@ class WalletNode:
         if end == 0:
             self.log.error("Error finding sub epoch")
             return False
-        all_peers_c = self.server.get_full_node_connections()
+        all_peers_c = self.server.get_connections(NodeType.FULL_NODE)
         all_peers = [(con, self.is_trusted(con)) for con in all_peers_c]
         blocks: Optional[List[HeaderBlock]] = await fetch_header_blocks_in_range(
             start, end, peer_request_cache, all_peers
         )
         if blocks is None:
-            self.log.error(f"Error fetching blocks {start} {end}")
+            if self._shut_down:
+                self.log.debug(f"Shutting down, block fetching from: {start} to {end} canceled.")
+            else:
+                self.log.error(f"Error fetching blocks {start} {end}")
             return False
 
         if compare_to_recent and weight_proof.recent_chain_data[0].header_hash != blocks[-1].header_hash:
@@ -1561,7 +1606,7 @@ class WalletNode:
             wallet_protocol.RequestPuzzleSolution(coin.name(), height)
         )
         if solution_response is None or not isinstance(solution_response, wallet_protocol.RespondPuzzleSolution):
-            raise ValueError(f"Was not able to obtain solution {solution_response}")
+            raise PeerRequestException(f"Was not able to obtain solution {solution_response}")
         assert solution_response.response.puzzle.get_tree_hash() == coin.puzzle_hash
         assert solution_response.response.coin_name == coin.name()
 
@@ -1576,7 +1621,8 @@ class WalletNode:
     ) -> List[CoinState]:
         msg = wallet_protocol.RegisterForCoinUpdates(coin_names, uint32(0))
         coin_state: Optional[RespondToCoinUpdates] = await peer.register_interest_in_coin(msg)
-        assert coin_state is not None
+        if coin_state is None or not isinstance(coin_state, wallet_protocol.RespondToCoinUpdates):
+            raise PeerRequestException(f"Was not able to get states for {coin_names}")
 
         if not self.is_trusted(peer):
             valid_list = []
@@ -1598,7 +1644,7 @@ class WalletNode:
             wallet_protocol.RequestChildren(coin_name)
         )
         if response is None or not isinstance(response, wallet_protocol.RespondChildren):
-            raise ValueError(f"Was not able to obtain children {response}")
+            raise PeerRequestException(f"Was not able to obtain children {response}")
 
         if not self.is_trusted(peer):
             request_cache = self.get_cache_for_peer(peer)
@@ -1616,6 +1662,6 @@ class WalletNode:
             ProtocolMessageTypes.send_transaction,
             wallet_protocol.SendTransaction(spend_bundle),
         )
-        full_nodes = self.server.get_full_node_connections()
+        full_nodes = self.server.get_connections(NodeType.FULL_NODE)
         for peer in full_nodes:
             await peer.send_message(msg)
