@@ -2,7 +2,7 @@ import pytest
 
 from typing import List, Tuple, Optional
 
-from blspy import AugSchemeMPL, G1Element, G2Element, PrivateKey
+from chinillablspy import AugSchemeMPL, G1Element, G2Element, PrivateKey
 
 from chinilla.types.blockchain_format.program import Program
 from chinilla.types.blockchain_format.sized_bytes import bytes32
@@ -87,7 +87,7 @@ class TestSingleton:
             # START TESTS
             # Generate starting info
             key_lookup = KeyTool()
-            pk: G1Element = public_key_for_index(1, key_lookup)
+            pk: G1Element = G1Element.from_bytes(public_key_for_index(1, key_lookup))
             starting_puzzle: Program = p2_delegated_puzzle_or_hidden_puzzle.puzzle_for_pk(pk)  # noqa
 
             if version == 0:
