@@ -1,5 +1,8 @@
 # flake8: noqa: E501
+from __future__ import annotations
+
 parallel = True
 job_timeout = 50
 check_resource_usage = True
 checkout_blocks_and_plots = True
+os_skip = ["windows"]
