@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from typing import List, Optional
 
 import pytest
@@ -17,7 +19,7 @@ from tests.blockchain.blockchain_test_utils import _validate_and_add_block
 coin_ids = [std_hash(i.to_bytes(4, "big")) for i in range(10)]
 phs = [std_hash(i.to_bytes(4, "big")) for i in range(10)]
 spends: List[Spend] = [
-    Spend(
+    Spend(  # type: ignore
         coin_ids[0],
         phs[0],
         None,
@@ -28,7 +30,7 @@ spends: List[Spend] = [
         ],
         [],
     ),
-    Spend(
+    Spend(  # type: ignore
         coin_ids[2],
         phs[0],
         None,
@@ -40,7 +42,7 @@ spends: List[Spend] = [
         ],
         [],
     ),
-    Spend(
+    Spend(  # type: ignore
         coin_ids[1],
         phs[7],
         None,
