@@ -1,13 +1,13 @@
 from typing import List
 
 import pytest
-from chiabip158 import PyBIP158
+from chinillabip158 import PyBIP158
 
 
 class TestFilter:
     @pytest.mark.asyncio
-    async def test_basic_filter_test(self, wallet_and_node, bt):
-        full_nodes, wallets = wallet_and_node
+    async def test_basic_filter_test(self, wallet_and_node):
+        full_nodes, wallets, bt = wallet_and_node
         wallet_node, server_2 = wallets[0]
         wallet = wallet_node.wallet_state_manager.main_wallet
 
