@@ -2,21 +2,19 @@
 from typing import List
 
 import pytest
-from blspy import AugSchemeMPL
+from chinillablspy import AugSchemeMPL
 
 from chinilla.consensus.pot_iterations import is_overflow_block
 from chinilla.full_node.signage_point import SignagePoint
 from chinilla.protocols import full_node_protocol
-from chinilla.rpc.full_node_rpc_api import FullNodeRpcApi
 from chinilla.rpc.full_node_rpc_client import FullNodeRpcClient
-from chinilla.rpc.rpc_server import start_rpc_server
 from chinilla.server.outbound_message import NodeType
 from chinilla.simulator.simulator_protocol import FarmNewBlockProtocol, ReorgProtocol
 from chinilla.types.full_block import FullBlock
 from chinilla.types.spend_bundle import SpendBundle
 from chinilla.types.unfinished_block import UnfinishedBlock
 from chinilla.util.hash import std_hash
-from chinilla.util.ints import uint8, uint16
+from chinilla.util.ints import uint8
 from chinilla.simulator.block_tools import get_signage_point
 from tests.blockchain.blockchain_test_utils import _validate_and_add_block
 from tests.connection_utils import connect_and_get_peer
@@ -28,37 +26,22 @@ from chinilla.simulator.wallet_tools import WalletTool
 
 class TestRpc:
     @pytest.mark.asyncio
-    async def test1(self, two_nodes_sim_and_wallets, self_hostname):
+    async def test1(self, two_nodes_sim_and_wallets_services, self_hostname):
         num_blocks = 5
-        nodes, _, bt = two_nodes_sim_and_wallets
-        full_node_api_1, full_node_api_2 = nodes
-        server_1 = full_node_api_1.full_node.server
+        nodes, _, bt = two_nodes_sim_and_wallets_services
+        full_node_service_1, full_node_service_2 = nodes
+        full_node_api_1 = full_node_service_1._api
+        full_node_api_2 = full_node_service_2._api
         server_2 = full_node_api_2.full_node.server
 
-        def stop_node_cb():
-            full_node_api_1._close()
-            server_1.close_all()
-
-        full_node_rpc_api = FullNodeRpcApi(full_node_api_1.full_node)
-
-        config = bt.config
-        hostname = config["self_hostname"]
-        daemon_port = config["daemon_port"]
-
-        rpc_cleanup, test_rpc_port = await start_rpc_server(
-            full_node_rpc_api,
-            hostname,
-            daemon_port,
-            uint16(0),
-            stop_node_cb,
-            bt.root_path,
-            config,
-            connect_to_daemon=False,
-        )
-
         try:
-            client = await FullNodeRpcClient.create(self_hostname, test_rpc_port, bt.root_path, config)
-            await validate_get_routes(client, full_node_rpc_api)
+            client = await FullNodeRpcClient.create(
+                self_hostname,
+                full_node_service_1.rpc_server.listen_port,
+                full_node_service_1.root_path,
+                full_node_service_1.config,
+            )
+            await validate_get_routes(client, full_node_service_1.rpc_server.rpc_api)
             state = await client.get_blockchain_state()
             assert state["peak"] is None
             assert not state["sync"]["sync_mode"]
@@ -174,7 +157,14 @@ class TestRpc:
 
             await full_node_api_1.farm_new_transaction_block(FarmNewBlockProtocol(ph_2))
 
-            assert (await client.get_coin_record_by_name(coin.name())).coin == coin
+            coin_record = await client.get_coin_record_by_name(coin.name())
+            assert coin_record.coin == coin
+            assert (
+                coin
+                in (
+                    await client.get_puzzle_and_solution(coin.parent_coin_info, coin_record.confirmed_block_index)
+                ).additions()
+            )
 
             assert len(await client.get_coin_records_by_puzzle_hash(ph_receiver)) == 1
             assert len(list(filter(lambda cr: not cr.spent, (await client.get_coin_records_by_puzzle_hash(ph))))) == 3
@@ -285,40 +275,28 @@ class TestRpc:
             # Checks that the RPC manages to stop the node
             client.close()
             await client.await_closed()
-            await rpc_cleanup()
 
     @pytest.mark.asyncio
-    async def test_signage_points(self, two_nodes_sim_and_wallets, empty_blockchain):
-        nodes, _, bt = two_nodes_sim_and_wallets
-        full_node_api_1, full_node_api_2 = nodes
+    async def test_signage_points(self, two_nodes_sim_and_wallets_services, empty_blockchain):
+        nodes, _, bt = two_nodes_sim_and_wallets_services
+        full_node_service_1, full_node_service_2 = nodes
+        full_node_api_1 = full_node_service_1._api
+        full_node_api_2 = full_node_service_2._api
         server_1 = full_node_api_1.full_node.server
         server_2 = full_node_api_2.full_node.server
 
         config = bt.config
         self_hostname = config["self_hostname"]
-        daemon_port = config["daemon_port"]
 
         peer = await connect_and_get_peer(server_1, server_2, self_hostname)
 
-        def stop_node_cb():
-            full_node_api_1._close()
-            server_1.close_all()
-
-        full_node_rpc_api = FullNodeRpcApi(full_node_api_1.full_node)
-
-        rpc_cleanup, test_rpc_port = await start_rpc_server(
-            full_node_rpc_api,
-            self_hostname,
-            daemon_port,
-            uint16(0),
-            stop_node_cb,
-            bt.root_path,
-            config,
-            connect_to_daemon=False,
-        )
-
         try:
-            client = await FullNodeRpcClient.create(self_hostname, test_rpc_port, bt.root_path, config)
+            client = await FullNodeRpcClient.create(
+                self_hostname,
+                full_node_service_1.rpc_server.listen_port,
+                full_node_service_1.root_path,
+                full_node_service_1.config,
+            )
 
             # Only provide one
             res = await client.get_recent_signage_point_or_eos(None, None)
@@ -429,4 +407,3 @@ class TestRpc:
             # Checks that the RPC manages to stop the node
             client.close()
             await client.await_closed()
-            await rpc_cleanup()
