@@ -1,7 +1,7 @@
 from typing import Any, Dict, Optional, List
 
 import pytest
-from blspy import G2Element
+from chinillablspy import G2Element
 
 from chinilla.types.announcement import Announcement
 from chinilla.types.blockchain_format.coin import Coin
@@ -12,11 +12,11 @@ from chinilla.types.mempool_inclusion_status import MempoolInclusionStatus
 from chinilla.types.spend_bundle import SpendBundle
 from chinilla.util.ints import uint64
 from chinilla.wallet.cat_wallet.cat_utils import (
-    CAT_MOD,
     construct_cat_puzzle,
     SpendableCAT,
     unsigned_spend_bundle_for_spendable_cats,
 )
+from chinilla.wallet.puzzles.cat_loader import CAT_MOD
 from chinilla.wallet.outer_puzzles import AssetType
 from chinilla.wallet.puzzle_drivers import PuzzleInfo
 from chinilla.wallet.payment import Payment
