@@ -1,7 +1,9 @@
+from __future__ import annotations
+
 import asyncio
 import logging
 from secrets import token_bytes
-from typing import Any, Callable, Optional
+from typing import Any, Callable, Coroutine, Optional
 
 import pytest
 
@@ -13,7 +15,7 @@ from chinilla.simulator.time_out_assert import time_out_assert, time_out_assert_
 from chinilla.types.blockchain_format.program import Program
 from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.peer_info import PeerInfo
-from chinilla.util.ints import uint16, uint32, uint64
+from chinilla.util.ints import uint16, uint32, uint64, uint128
 from chinilla.wallet.cat_wallet.cat_wallet import CATWallet
 from chinilla.wallet.did_wallet.did_wallet import DIDWallet
 from chinilla.wallet.nft_wallet.nft_wallet import NFTWallet
@@ -21,6 +23,7 @@ from chinilla.wallet.outer_puzzles import create_asset_id, match_puzzle
 from chinilla.wallet.puzzle_drivers import PuzzleInfo
 from chinilla.wallet.trading.offer import Offer
 from chinilla.wallet.trading.trade_status import TradeStatus
+from chinilla.wallet.uncurried_puzzle import uncurry_puzzle
 from chinilla.wallet.util.compute_memos import compute_memos
 
 # from clvm_tools.binutils import disassemble
@@ -40,15 +43,21 @@ def mempool_not_empty(fnapi: FullNodeSimulator) -> bool:
     return len(fnapi.full_node.mempool_manager.mempool.spends) > 0
 
 
-async def farm_blocks_until(predicate_f: Callable[[], bool], fnapi: FullNodeSimulator, ph: bytes32) -> None:
+async def farm_blocks_until(
+    predicate_f: Callable[[], Coroutine[Any, Any, bool]], fnapi: FullNodeSimulator, ph: bytes32
+) -> None:
     for i in range(50):
         await fnapi.farm_new_transaction_block(FarmNewBlockProtocol(ph))
-        if predicate_f():
+        if await predicate_f():
             return None
         await asyncio.sleep(0.3)
     raise TimeoutError()
 
 
+async def get_nft_count(wallet: NFTWallet) -> int:
+    return len(await wallet.get_current_nfts())
+
+
 @pytest.mark.parametrize(
     "trusted",
     [True, False],
@@ -142,7 +151,7 @@ async def test_nft_offer_sell_nft(two_wallet_nodes: Any, trusted: Any) -> None:
     await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_token))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node_api)
 
-    await time_out_assert(20, len, 1, nft_wallet_maker.my_nft_coins)
+    await time_out_assert(20, get_nft_count, 1, nft_wallet_maker)
 
     # TAKER SETUP -  NO DID
     nft_wallet_taker = await NFTWallet.create_new_nft_wallet(
@@ -153,13 +162,13 @@ async def test_nft_offer_sell_nft(two_wallet_nodes: Any, trusted: Any) -> None:
     trade_manager_maker = wallet_maker.wallet_state_manager.trade_manager
     trade_manager_taker = wallet_taker.wallet_state_manager.trade_manager
 
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     assert len(coins_maker) == 1
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_taker) == 0
 
     nft_to_offer = coins_maker[0]
-    nft_to_offer_info: Optional[PuzzleInfo] = match_puzzle(nft_to_offer.full_puzzle)
+    nft_to_offer_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_offer.full_puzzle))
     nft_to_offer_asset_id: bytes32 = create_asset_id(nft_to_offer_info)  # type: ignore
     hcx_requested = 1000
     maker_fee = uint64(433)
@@ -189,14 +198,16 @@ async def test_nft_offer_sell_nft(two_wallet_nodes: Any, trusted: Any) -> None:
     assert success is True
     assert trade_take is not None
 
-    def maker_0_taker_1() -> bool:
-        return len(nft_wallet_maker.my_nft_coins) == 0 and len(nft_wallet_taker.my_nft_coins) == 1
+    async def maker_0_taker_1() -> bool:
+        return (
+            len(await nft_wallet_maker.get_current_nfts()) == 0 and len(await nft_wallet_taker.get_current_nfts()) == 1
+        )
 
     await farm_blocks_until(maker_0_taker_1, full_node_api, ph_token)
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node_api)
 
-    await time_out_assert(20, len, 0, nft_wallet_maker.my_nft_coins)
-    await time_out_assert(20, len, 1, nft_wallet_taker.my_nft_coins)
+    await time_out_assert(20, get_nft_count, 0, nft_wallet_maker)
+    await time_out_assert(20, get_nft_count, 1, nft_wallet_taker)
 
     # assert payments and royalties
     expected_royalty = uint64(hcx_requested * royalty_basis_pts / 10000)
@@ -299,7 +310,7 @@ async def test_nft_offer_request_nft(two_wallet_nodes: Any, trusted: Any) -> Non
     await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_token))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node_api)
 
-    await time_out_assert(20, len, 1, nft_wallet_taker.my_nft_coins)
+    await time_out_assert(20, get_nft_count, 1, nft_wallet_taker)
 
     # MAKER SETUP -  NO DID
     nft_wallet_maker = await NFTWallet.create_new_nft_wallet(
@@ -310,13 +321,13 @@ async def test_nft_offer_request_nft(two_wallet_nodes: Any, trusted: Any) -> Non
     trade_manager_maker = wallet_maker.wallet_state_manager.trade_manager
     trade_manager_taker = wallet_taker.wallet_state_manager.trade_manager
 
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     assert len(coins_maker) == 0
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_taker) == 1
 
     nft_to_request = coins_taker[0]
-    nft_to_request_info: Optional[PuzzleInfo] = match_puzzle(nft_to_request.full_puzzle)
+    nft_to_request_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_request.full_puzzle))
 
     assert isinstance(nft_to_request_info, PuzzleInfo)
     nft_to_request_asset_id = create_asset_id(nft_to_request_info)
@@ -347,8 +358,10 @@ async def test_nft_offer_request_nft(two_wallet_nodes: Any, trusted: Any) -> Non
     await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_token))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node_api)
 
-    def maker_1_taker_0() -> bool:
-        return len(nft_wallet_maker.my_nft_coins) == 1 and len(nft_wallet_taker.my_nft_coins) == 0
+    async def maker_1_taker_0() -> bool:
+        return (
+            len(await nft_wallet_maker.get_current_nfts()) == 1 and len(await nft_wallet_taker.get_current_nfts()) == 0
+        )
 
     await farm_blocks_until(maker_1_taker_0, full_node_api, ph_token)
 
@@ -454,7 +467,7 @@ async def test_nft_offer_sell_did_to_did(two_wallet_nodes: Any, trusted: Any) ->
     await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(bytes32([0] * 32)))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node_api)
 
-    await time_out_assert(20, len, 1, nft_wallet_maker.my_nft_coins)
+    await time_out_assert(20, get_nft_count, 1, nft_wallet_maker)
 
     # TAKER SETUP -  WITH DID
     did_wallet_taker: DIDWallet = await DIDWallet.create_new_did_wallet(
@@ -485,13 +498,13 @@ async def test_nft_offer_sell_did_to_did(two_wallet_nodes: Any, trusted: Any) ->
     trade_manager_maker = wallet_maker.wallet_state_manager.trade_manager
     trade_manager_taker = wallet_taker.wallet_state_manager.trade_manager
 
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     assert len(coins_maker) == 1
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_taker) == 0
 
     nft_to_offer = coins_maker[0]
-    nft_to_offer_info: Optional[PuzzleInfo] = match_puzzle(nft_to_offer.full_puzzle)
+    nft_to_offer_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_offer.full_puzzle))
     nft_to_offer_asset_id: bytes32 = create_asset_id(nft_to_offer_info)  # type: ignore
     hcx_requested = 1000
     maker_fee = uint64(433)
@@ -518,20 +531,20 @@ async def test_nft_offer_sell_did_to_did(two_wallet_nodes: Any, trusted: Any) ->
     assert success is True
     assert trade_take is not None
 
-    def maker_0_taker_1() -> bool:
+    async def maker_0_taker_1() -> bool:
         return (
-            len(nft_wallet_maker.my_nft_coins) == 0
+            len(await nft_wallet_maker.get_current_nfts()) == 0
             and len(wallet_taker.wallet_state_manager.wallets) == 4
-            and len(wallet_taker.wallet_state_manager.wallets[4].my_nft_coins) == 1
+            and len(await wallet_taker.wallet_state_manager.wallets[4].get_current_nfts()) == 1
         )
 
     await farm_blocks_until(maker_0_taker_1, full_node_api, ph_token)
 
-    await time_out_assert(20, len, 0, nft_wallet_maker.my_nft_coins)
+    await time_out_assert(20, get_nft_count, 0, nft_wallet_maker)
     # assert nnew nft wallet is created for taker
     await time_out_assert(20, len, 4, wallet_taker.wallet_state_manager.wallets)
-    await time_out_assert(20, len, 1, wallet_taker.wallet_state_manager.wallets[4].my_nft_coins)
-    assert wallet_taker.wallet_state_manager.wallets[4].my_nft_coins[0].nft_id == nft_to_offer_asset_id
+    await time_out_assert(20, get_nft_count, 1, wallet_taker.wallet_state_manager.wallets[4])
+    assert (await wallet_taker.wallet_state_manager.wallets[4].get_current_nfts())[0].nft_id == nft_to_offer_asset_id
     # assert payments and royalties
     expected_royalty = uint64(hcx_requested * royalty_basis_pts / 10000)
     expected_maker_balance = funds - 2 - maker_fee + hcx_requested + expected_royalty
@@ -633,7 +646,7 @@ async def test_nft_offer_sell_nft_for_cat(two_wallet_nodes: Any, trusted: Any) -
     await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_token))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node_api)
 
-    await time_out_assert(20, len, 1, nft_wallet_maker.my_nft_coins)
+    await time_out_assert(20, get_nft_count, 1, nft_wallet_maker)
 
     # TAKER SETUP -  NO DID
     nft_wallet_taker = await NFTWallet.create_new_nft_wallet(
@@ -644,9 +657,9 @@ async def test_nft_offer_sell_nft_for_cat(two_wallet_nodes: Any, trusted: Any) -
     trade_manager_maker = wallet_maker.wallet_state_manager.trade_manager
     trade_manager_taker = wallet_taker.wallet_state_manager.trade_manager
 
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     assert len(coins_maker) == 1
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_taker) == 0
 
     # Create new CAT and wallets for maker and taker
@@ -690,7 +703,7 @@ async def test_nft_offer_sell_nft_for_cat(two_wallet_nodes: Any, trusted: Any) -
     await time_out_assert(20, cat_wallet_maker.get_confirmed_balance, maker_cat_balance)
     await time_out_assert(20, cat_wallet_taker.get_confirmed_balance, taker_cat_balance)
     nft_to_offer = coins_maker[0]
-    nft_to_offer_info: Optional[PuzzleInfo] = match_puzzle(nft_to_offer.full_puzzle)
+    nft_to_offer_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_offer.full_puzzle))
     nft_to_offer_asset_id: bytes32 = create_asset_id(nft_to_offer_info)  # type: ignore
     cats_requested = 1000
     maker_fee = uint64(433)
@@ -717,8 +730,10 @@ async def test_nft_offer_sell_nft_for_cat(two_wallet_nodes: Any, trusted: Any) -
     assert success is True
     assert trade_take is not None
 
-    def maker_0_taker_1() -> bool:
-        return len(nft_wallet_maker.my_nft_coins) == 0 and len(nft_wallet_taker.my_nft_coins) == 1
+    async def maker_0_taker_1() -> bool:
+        return (
+            len(await nft_wallet_maker.get_current_nfts()) == 0 and len(await nft_wallet_taker.get_current_nfts()) == 1
+        )
 
     await farm_blocks_until(maker_0_taker_1, full_node_api, ph_token)
 
@@ -825,7 +840,7 @@ async def test_nft_offer_request_nft_for_cat(two_wallet_nodes: Any, trusted: boo
     await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_token))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node_api)
 
-    await time_out_assert(20, len, 1, nft_wallet_taker.my_nft_coins)
+    await time_out_assert(20, get_nft_count, 1, nft_wallet_taker)
 
     # MAKER SETUP -  NO DID
     nft_wallet_maker = await NFTWallet.create_new_nft_wallet(
@@ -836,9 +851,9 @@ async def test_nft_offer_request_nft_for_cat(two_wallet_nodes: Any, trusted: boo
     trade_manager_maker = wallet_maker.wallet_state_manager.trade_manager
     trade_manager_taker = wallet_taker.wallet_state_manager.trade_manager
 
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     assert len(coins_maker) == 0
-    coins_taker = nft_wallet_taker.my_nft_coins
+    coins_taker = await nft_wallet_taker.get_current_nfts()
     assert len(coins_taker) == 1
 
     # Create new CAT and wallets for maker and taker
@@ -892,7 +907,7 @@ async def test_nft_offer_request_nft_for_cat(two_wallet_nodes: Any, trusted: boo
     await time_out_assert(15, cat_wallet_taker.get_confirmed_balance, taker_cat_balance)
 
     nft_to_request = coins_taker[0]
-    nft_to_request_info: Optional[PuzzleInfo] = match_puzzle(nft_to_request.full_puzzle)
+    nft_to_request_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_request.full_puzzle))
     nft_to_request_asset_id: bytes32 = create_asset_id(nft_to_request_info)  # type: ignore
     cats_requested = 10000
     maker_fee = uint64(433)
@@ -918,8 +933,10 @@ async def test_nft_offer_request_nft_for_cat(two_wallet_nodes: Any, trusted: boo
     assert success is True
     assert trade_take is not None
 
-    def maker_1_taker_0() -> bool:
-        return len(nft_wallet_maker.my_nft_coins) == 1 and len(nft_wallet_taker.my_nft_coins) == 0
+    async def maker_1_taker_0() -> bool:
+        return (
+            len(await nft_wallet_maker.get_current_nfts()) == 1 and len(await nft_wallet_taker.get_current_nfts()) == 0
+        )
 
     await farm_blocks_until(maker_1_taker_0, full_node_api, ph_token)
 
@@ -998,7 +1015,7 @@ async def test_nft_offer_sell_cancel(two_wallet_nodes: Any, trusted: Any) -> Non
     )
     metadata = Program.to(
         [
-            ("u", ["https://www.chinilla.net/img/branding/chinilla-logo.svg"]),
+            ("u", ["https://www.chinilla.com/img/branding/chinilla-logo.svg"]),
             ("h", "0xD4584AD463139FA8C0D9F68F4B59F185"),
         ]
     )
@@ -1017,16 +1034,16 @@ async def test_nft_offer_sell_cancel(two_wallet_nodes: Any, trusted: Any) -> Non
     await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_token))
     await time_out_assert(20, wallets_are_synced, True, [wallet_node_maker], full_node_api)
 
-    await time_out_assert(20, len, 1, nft_wallet_maker.my_nft_coins)
+    await time_out_assert(20, get_nft_count, 1, nft_wallet_maker)
 
     # maker create offer: NFT for hcx
     trade_manager_maker = wallet_maker.wallet_state_manager.trade_manager
 
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     assert len(coins_maker) == 1
 
     nft_to_offer = coins_maker[0]
-    nft_to_offer_info: Optional[PuzzleInfo] = match_puzzle(nft_to_offer.full_puzzle)
+    nft_to_offer_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_offer.full_puzzle))
     nft_to_offer_asset_id: bytes32 = create_asset_id(nft_to_offer_info)  # type: ignore
     hcx_requested = 1000
     maker_fee = uint64(433)
@@ -1138,16 +1155,16 @@ async def test_nft_offer_sell_cancel_in_batch(two_wallet_nodes: Any, trusted: An
     for i in range(1, num_blocks):
         await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_token))
 
-    await time_out_assert(10, len, 1, nft_wallet_maker.my_nft_coins)
+    await time_out_assert(10, get_nft_count, 1, nft_wallet_maker)
 
     # maker create offer: NFT for hcx
     trade_manager_maker = wallet_maker.wallet_state_manager.trade_manager
 
-    coins_maker = nft_wallet_maker.my_nft_coins
+    coins_maker = await nft_wallet_maker.get_current_nfts()
     assert len(coins_maker) == 1
 
     nft_to_offer = coins_maker[0]
-    nft_to_offer_info: Optional[PuzzleInfo] = match_puzzle(nft_to_offer.full_puzzle)
+    nft_to_offer_info: Optional[PuzzleInfo] = match_puzzle(uncurry_puzzle(nft_to_offer.full_puzzle))
     nft_to_offer_asset_id: bytes32 = create_asset_id(nft_to_offer_info)  # type: ignore
     hcx_requested = 1000
     maker_fee = uint64(433)
@@ -1174,3 +1191,300 @@ async def test_nft_offer_sell_cancel_in_batch(two_wallet_nodes: Any, trusted: An
         await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(bytes32([0] * 32)))
 
     await time_out_assert(15, get_trade_and_status, TradeStatus.CANCELLED, trade_manager_maker, trade_make)
+
+
+@pytest.mark.parametrize(
+    "trusted",
+    [True, False],
+)
+@pytest.mark.asyncio
+# @pytest.mark.skip
+async def test_complex_nft_offer(two_wallet_nodes: Any, trusted: Any) -> None:
+    """
+    This test is going to create an offer where the maker offers 1 NFT and 1 CAT for 2 NFTs, an HCX and a CAT
+    """
+    full_nodes, wallets, _ = two_wallet_nodes
+    full_node_api: FullNodeSimulator = full_nodes[0]
+    full_node_server = full_node_api.server
+    wallet_node_maker, server_0 = wallets[0]
+    wallet_node_taker, server_1 = wallets[1]
+    wsm_maker = wallet_node_maker.wallet_state_manager
+    wsm_taker = wallet_node_taker.wallet_state_manager
+    wallet_maker = wsm_maker.main_wallet
+    wallet_taker = wsm_taker.main_wallet
+
+    ph_maker = await wallet_maker.get_new_puzzlehash()
+    ph_taker = await wallet_taker.get_new_puzzlehash()
+    ph_token = bytes32(token_bytes())
+
+    if trusted:
+        wallet_node_maker.config["trusted_peers"] = {
+            full_node_api.full_node.server.node_id.hex(): full_node_api.full_node.server.node_id.hex()
+        }
+        wallet_node_taker.config["trusted_peers"] = {
+            full_node_api.full_node.server.node_id.hex(): full_node_api.full_node.server.node_id.hex()
+        }
+    else:
+        wallet_node_maker.config["trusted_peers"] = {}
+        wallet_node_taker.config["trusted_peers"] = {}
+    wallet_node_maker.config["automatically_add_unknown_cats"] = True
+    wallet_node_taker.config["automatically_add_unknown_cats"] = True
+
+    await server_0.start_client(PeerInfo("localhost", uint16(full_node_server._port)), None)
+    await server_1.start_client(PeerInfo("localhost", uint16(full_node_server._port)), None)
+
+    # Need money for fees and offering
+    for i in range(0, 2):
+        await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_maker))
+        await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_taker))
+    await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_taker))
+    await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_token))
+    await time_out_assert(30, wallets_are_synced, True, [wallet_node_maker, wallet_node_taker], full_node_api)
+
+    funds_maker = sum([calculate_pool_reward(uint32(i)) + calculate_base_farmer_reward(uint32(i)) for i in range(1, 3)])
+    funds_taker = sum([calculate_pool_reward(uint32(i)) + calculate_base_farmer_reward(uint32(i)) for i in range(1, 4)])
+
+    await time_out_assert(30, wallet_maker.get_unconfirmed_balance, funds_maker)
+    await time_out_assert(30, wallet_maker.get_confirmed_balance, funds_maker)
+    await time_out_assert(30, wallet_taker.get_unconfirmed_balance, funds_taker)
+    await time_out_assert(30, wallet_taker.get_confirmed_balance, funds_taker)
+
+    CAT_AMOUNT = uint64(100000000)
+    async with wsm_maker.lock:
+        cat_wallet_maker: CATWallet = await CATWallet.create_new_cat_wallet(
+            wsm_maker, wallet_maker, {"identifier": "genesis_by_id"}, CAT_AMOUNT
+        )
+    async with wsm_maker.lock:
+        cat_wallet_taker: CATWallet = await CATWallet.create_new_cat_wallet(
+            wsm_taker, wallet_taker, {"identifier": "genesis_by_id"}, CAT_AMOUNT
+        )
+    cat_spend_bundle_maker = (
+        await wallet_node_maker.wallet_state_manager.tx_store.get_unconfirmed_for_wallet(wallet_maker.id())
+    )[0].spend_bundle
+    cat_spend_bundle_taker = (
+        await wallet_node_taker.wallet_state_manager.tx_store.get_unconfirmed_for_wallet(wallet_taker.id())
+    )[0].spend_bundle
+    await time_out_assert_not_none(
+        5, full_node_api.full_node.mempool_manager.get_spendbundle, cat_spend_bundle_maker.name()
+    )
+    await time_out_assert_not_none(
+        5, full_node_api.full_node.mempool_manager.get_spendbundle, cat_spend_bundle_taker.name()
+    )
+
+    # We'll need these later
+    basic_nft_wallet_maker = await NFTWallet.create_new_nft_wallet(wsm_maker, wallet_maker, name="NFT WALLET MAKER")
+    basic_nft_wallet_taker = await NFTWallet.create_new_nft_wallet(wsm_taker, wallet_taker, name="NFT WALLET TAKER")
+
+    did_wallet_maker: DIDWallet = await DIDWallet.create_new_did_wallet(wsm_maker, wallet_maker, uint64(1))
+    did_wallet_taker: DIDWallet = await DIDWallet.create_new_did_wallet(wsm_taker, wallet_taker, uint64(1))
+    did_spend_bundle_maker = (
+        await wallet_node_maker.wallet_state_manager.tx_store.get_unconfirmed_for_wallet(did_wallet_maker.id())
+    )[0].spend_bundle
+    did_spend_bundle_taker = (
+        await wallet_node_taker.wallet_state_manager.tx_store.get_unconfirmed_for_wallet(did_wallet_taker.id())
+    )[0].spend_bundle
+
+    await time_out_assert_not_none(
+        5, full_node_api.full_node.mempool_manager.get_spendbundle, did_spend_bundle_maker.name()
+    )
+    await time_out_assert_not_none(
+        5, full_node_api.full_node.mempool_manager.get_spendbundle, did_spend_bundle_taker.name()
+    )
+
+    await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_token))
+
+    funds_maker = funds_maker - 1 - CAT_AMOUNT
+    funds_taker = funds_taker - 1 - CAT_AMOUNT
+
+    await time_out_assert(30, wallet_maker.get_unconfirmed_balance, funds_maker)
+    await time_out_assert(30, wallet_maker.get_confirmed_balance, funds_maker)
+    await time_out_assert(30, wallet_taker.get_unconfirmed_balance, funds_taker)
+    await time_out_assert(30, wallet_taker.get_confirmed_balance, funds_taker)
+    await time_out_assert(30, cat_wallet_maker.get_confirmed_balance, CAT_AMOUNT)
+    await time_out_assert(30, cat_wallet_maker.get_unconfirmed_balance, CAT_AMOUNT)
+    await time_out_assert(30, cat_wallet_taker.get_confirmed_balance, CAT_AMOUNT)
+    await time_out_assert(30, cat_wallet_taker.get_unconfirmed_balance, CAT_AMOUNT)
+
+    did_id_maker = bytes32.fromhex(did_wallet_maker.get_my_DID())
+    did_id_taker = bytes32.fromhex(did_wallet_taker.get_my_DID())
+    target_puzhash_maker = ph_maker
+    target_puzhash_taker = ph_taker
+    royalty_puzhash_maker = ph_maker
+    royalty_puzhash_taker = ph_taker
+    royalty_basis_pts_maker = uint16(200)
+    royalty_basis_pts_taker_1 = uint16(500)
+    royalty_basis_pts_taker_2 = uint16(100)
+
+    nft_wallet_maker = await NFTWallet.create_new_nft_wallet(
+        wallet_node_maker.wallet_state_manager, wallet_maker, name="NFT WALLET DID 1", did_id=did_id_maker
+    )
+    nft_wallet_taker = await NFTWallet.create_new_nft_wallet(
+        wallet_node_taker.wallet_state_manager, wallet_taker, name="NFT WALLET DID 1", did_id=did_id_taker
+    )
+    metadata = Program.to(
+        [
+            ("u", ["https://www.chinilla.com/img/branding/chinilla-logo.svg"]),
+            ("h", "0xD4584AD463139FA8C0D9F68F4B59F185"),
+        ]
+    )
+
+    sb_maker = await nft_wallet_maker.generate_new_nft(
+        metadata,
+        target_puzhash_maker,
+        royalty_puzhash_maker,
+        royalty_basis_pts_maker,
+        did_id_maker,
+    )
+    sb_taker_1 = await nft_wallet_taker.generate_new_nft(
+        metadata,
+        target_puzhash_taker,
+        royalty_puzhash_taker,
+        royalty_basis_pts_taker_1,
+        did_id_taker,
+    )
+    assert sb_maker is not None
+    assert sb_taker_1 is not None
+    await time_out_assert_not_none(10, full_node_api.full_node.mempool_manager.get_spendbundle, sb_maker.name())
+    await time_out_assert_not_none(10, full_node_api.full_node.mempool_manager.get_spendbundle, sb_taker_1.name())
+    await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_token))
+
+    funds_maker -= 1
+    funds_taker -= 1
+
+    await time_out_assert(30, wallet_maker.get_unconfirmed_balance, funds_maker)
+    await time_out_assert(30, wallet_maker.get_confirmed_balance, funds_maker)
+    await time_out_assert(30, wallet_taker.get_unconfirmed_balance, funds_taker)
+    await time_out_assert(30, wallet_taker.get_confirmed_balance, funds_taker)
+    await time_out_assert(30, get_nft_count, 1, nft_wallet_maker)
+    await time_out_assert(30, get_nft_count, 1, nft_wallet_taker)
+
+    # MAke one more NFT for the taker
+    sb_taker_2 = await nft_wallet_taker.generate_new_nft(
+        metadata,
+        target_puzhash_taker,
+        royalty_puzhash_taker,
+        royalty_basis_pts_taker_2,
+        did_id_taker,
+    )
+    assert sb_taker_2 is not None
+    await time_out_assert_not_none(10, full_node_api.full_node.mempool_manager.get_spendbundle, sb_taker_2.name())
+    await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_token))
+
+    funds_taker -= 1
+
+    await time_out_assert(30, wallet_taker.get_unconfirmed_balance, funds_taker)
+    await time_out_assert(30, wallet_taker.get_confirmed_balance, funds_taker)
+    await time_out_assert(30, get_nft_count, 2, nft_wallet_taker)
+
+    trade_manager_maker = wsm_maker.trade_manager
+    trade_manager_taker = wsm_taker.trade_manager
+    maker_nfts = await nft_wallet_maker.get_current_nfts()
+    taker_nfts = await nft_wallet_taker.get_current_nfts()
+    nft_to_offer_asset_id_maker: bytes32 = maker_nfts[0].nft_id
+    nft_to_offer_asset_id_taker_1: bytes32 = taker_nfts[0].nft_id
+    nft_to_offer_asset_id_taker_2: bytes32 = taker_nfts[1].nft_id
+    HCX_REQUESTED = 2000000000000
+    CAT_REQUESTED = 100000
+    FEE = uint64(2000000000000)
+
+    complex_nft_offer = {
+        nft_to_offer_asset_id_maker: -1,
+        cat_wallet_maker.id(): CAT_REQUESTED * -1,
+        1: HCX_REQUESTED,
+        nft_to_offer_asset_id_taker_1: 1,
+        nft_to_offer_asset_id_taker_2: 1,
+        bytes32.from_hexstr(cat_wallet_taker.get_asset_id()): CAT_REQUESTED,
+    }
+
+    driver_dict = {
+        nft_to_offer_asset_id_taker_1: match_puzzle(uncurry_puzzle(taker_nfts[0].full_puzzle)),
+        nft_to_offer_asset_id_taker_2: match_puzzle(uncurry_puzzle(taker_nfts[1].full_puzzle)),
+        bytes32.from_hexstr(cat_wallet_taker.get_asset_id()): PuzzleInfo(
+            {
+                "type": "CAT",
+                "tail": "0x" + cat_wallet_taker.get_asset_id(),
+            }
+        ),
+    }
+
+    success, trade_make, error = await trade_manager_maker.create_offer_for_ids(
+        complex_nft_offer, driver_dict=driver_dict, fee=FEE
+    )
+    assert error is None
+    assert success
+    assert trade_make is not None
+
+    success, trade_take, error = await trade_manager_taker.respond_to_offer(
+        Offer.from_bytes(trade_make.offer),
+        wallet_node_taker.get_full_node_peer(),
+        fee=FEE,
+    )
+    assert error is None
+    assert success
+    assert trade_take is not None
+    await time_out_assert(20, mempool_not_empty, True, full_node_api)
+
+    await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph_token))
+
+    # Now let's make sure the final wallet state is correct
+    maker_royalty_summary = NFTWallet.royalty_calculation(
+        {
+            nft_to_offer_asset_id_maker: (royalty_puzhash_maker, royalty_basis_pts_maker),
+        },
+        {
+            None: uint64(HCX_REQUESTED),
+            bytes32.from_hexstr(cat_wallet_taker.get_asset_id()): uint64(CAT_REQUESTED),
+        },
+    )
+    taker_royalty_summary = NFTWallet.royalty_calculation(
+        {
+            nft_to_offer_asset_id_taker_1: (royalty_puzhash_taker, royalty_basis_pts_taker_1),
+            nft_to_offer_asset_id_taker_2: (royalty_puzhash_taker, royalty_basis_pts_taker_2),
+        },
+        {
+            bytes32.from_hexstr(cat_wallet_maker.get_asset_id()): uint64(CAT_REQUESTED),
+        },
+    )
+    maker_hcx_royalties_expected = maker_royalty_summary[nft_to_offer_asset_id_maker][0]["amount"]
+    maker_cat_royalties_expected = maker_royalty_summary[nft_to_offer_asset_id_maker][1]["amount"]
+    taker_cat_royalties_expected = (
+        taker_royalty_summary[nft_to_offer_asset_id_taker_1][0]["amount"]
+        + taker_royalty_summary[nft_to_offer_asset_id_taker_2][0]["amount"]
+    )
+    funds_maker = int(funds_maker - FEE + HCX_REQUESTED + maker_hcx_royalties_expected)
+    funds_taker = int(funds_taker - FEE - HCX_REQUESTED - maker_hcx_royalties_expected)
+
+    await time_out_assert(30, wallet_maker.get_unconfirmed_balance, funds_maker)
+    await time_out_assert(30, wallet_maker.get_confirmed_balance, funds_maker)
+    await time_out_assert(30, wallet_taker.get_unconfirmed_balance, funds_taker)
+    await time_out_assert(30, wallet_taker.get_confirmed_balance, funds_taker)
+
+    async def get_cat_wallet_and_check_balance(asset_id: str, wsm: Any) -> uint128:
+        cat_wallet = await wsm.get_wallet_for_asset_id(asset_id)
+        if cat_wallet is None:
+            return uint128(0)
+        else:
+            return uint128(await cat_wallet.get_confirmed_balance())
+
+    await time_out_assert(
+        30,
+        get_cat_wallet_and_check_balance,
+        CAT_REQUESTED + maker_cat_royalties_expected,
+        cat_wallet_taker.get_asset_id(),
+        wsm_maker,
+    )
+    await time_out_assert(
+        30,
+        get_cat_wallet_and_check_balance,
+        CAT_REQUESTED + taker_cat_royalties_expected,
+        cat_wallet_maker.get_asset_id(),
+        wsm_taker,
+    )
+    maker_nfts = await basic_nft_wallet_maker.get_current_nfts()
+    taker_nfts = await basic_nft_wallet_taker.get_current_nfts()
+    await time_out_assert(30, len, 2, maker_nfts)
+    await time_out_assert(30, len, 1, taker_nfts)
+
+    assert nft_to_offer_asset_id_maker == taker_nfts[0].nft_id
+    assert nft_to_offer_asset_id_taker_1 in [nft.nft_id for nft in maker_nfts]
+    assert nft_to_offer_asset_id_taker_2 in [nft.nft_id for nft in maker_nfts]
