@@ -1,8 +1,10 @@
+from __future__ import annotations
+
 import itertools
 from typing import List, Tuple
 
 import pytest
-from blspy import G2Element
+from chinillablspy import G2Element
 
 from chinilla.clvm.spend_sim import SimClient, SpendSim
 from chinilla.types.announcement import Announcement
