@@ -22,11 +22,11 @@ from chinilla.types.spend_bundle import SpendBundle
 from chinilla.util.ints import uint16, uint32, uint64
 from chinilla.wallet.wallet import Wallet
 from chinilla.wallet.wallet_state_manager import WalletStateManager
+from tests.util.wallet_is_synced import wallet_is_synced
 from tests.connection_utils import add_dummy_connection
-from tests.pools.test_pool_rpc import wallet_is_synced
-from tests.time_out_assert import time_out_assert
+from chinilla.simulator.time_out_assert import time_out_assert
 from tests.wallet.cat_wallet.test_cat_wallet import tx_in_pool
-from tests.wallet_tools import WalletTool
+from chinilla.simulator.wallet_tools import WalletTool
 
 
 def wallet_height_at_least(wallet_node, h):
@@ -52,7 +52,7 @@ class TestSimpleSyncProtocol:
     @pytest.mark.asyncio
     async def test_subscribe_for_ph(self, wallet_node_simulator, self_hostname):
         num_blocks = 4
-        full_nodes, wallets = wallet_node_simulator
+        full_nodes, wallets, _ = wallet_node_simulator
         full_node_api = full_nodes[0]
         wallet_node, server_2 = wallets[0]
         fn_server = full_node_api.full_node.server
@@ -171,7 +171,7 @@ class TestSimpleSyncProtocol:
             for cr in await full_node_api.full_node.coin_store.get_coin_records_by_puzzle_hash(False, puzzle_hash)
         )
 
-        await time_out_assert(15, wallet.get_confirmed_balance, funds)
+        await time_out_assert(20, wallet.get_confirmed_balance, funds)
         assert funds == fn_amount
 
         msg_1 = wallet_protocol.RegisterForPhUpdates([puzzle_hash], 0)
@@ -180,7 +180,7 @@ class TestSimpleSyncProtocol:
         data_response_1: RespondToPhUpdates = RespondToCoinUpdates.from_bytes(msg_response_1.data)
         assert len(data_response_1.coin_states) == 2 * num_blocks  # 2 per height farmer / pool reward
 
-        await time_out_assert(10, wallet_is_synced, True, wallet_node, full_node_api)
+        await time_out_assert(20, wallet_is_synced, True, wallet_node, full_node_api)
         tx_record = await wallet.generate_signed_transaction(uint64(10), puzzle_hash, uint64(0))
         assert len(tx_record.spend_bundle.removals()) == 1
         spent_coin = tx_record.spend_bundle.removals()[0]
@@ -198,7 +198,7 @@ class TestSimpleSyncProtocol:
         # Let's make sure the wallet can handle a non ephemeral launcher
         from chinilla.wallet.puzzles.singleton_top_layer import SINGLETON_LAUNCHER_HASH
 
-        await time_out_assert(10, wallet_is_synced, True, wallet_node, full_node_api)
+        await time_out_assert(20, wallet_is_synced, True, wallet_node, full_node_api)
         tx_record = await wallet.generate_signed_transaction(uint64(10), SINGLETON_LAUNCHER_HASH, uint64(0))
         await wallet.push_transaction(tx_record)
 
@@ -209,7 +209,7 @@ class TestSimpleSyncProtocol:
         for i in range(0, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(SINGLETON_LAUNCHER_HASH))
 
-        await time_out_assert(10, wallet_is_synced, True, wallet_node, full_node_api)
+        await time_out_assert(20, wallet_is_synced, True, wallet_node, full_node_api)
 
         # Send a transaction to make sure the wallet is still running
         tx_record = await wallet.generate_signed_transaction(uint64(10), junk_ph, uint64(0))
@@ -240,7 +240,7 @@ class TestSimpleSyncProtocol:
     @pytest.mark.asyncio
     async def test_subscribe_for_coin_id(self, wallet_node_simulator, self_hostname):
         num_blocks = 4
-        full_nodes, wallets = wallet_node_simulator
+        full_nodes, wallets, _ = wallet_node_simulator
         full_node_api = full_nodes[0]
         wallet_node, server_2 = wallets[0]
         fn_server = full_node_api.full_node.server
@@ -261,7 +261,7 @@ class TestSimpleSyncProtocol:
             [calculate_pool_reward(uint32(i)) + calculate_base_farmer_reward(uint32(i)) for i in range(1, num_blocks)]
         )
 
-        await time_out_assert(15, standard_wallet.get_confirmed_balance, funds)
+        await time_out_assert(20, standard_wallet.get_confirmed_balance, funds)
 
         my_coins: List[CoinRecord] = await full_node_api.full_node.coin_store.get_coin_records_by_puzzle_hash(
             True, puzzle_hash
@@ -301,7 +301,7 @@ class TestSimpleSyncProtocol:
         assert notified_coins == coins
 
         # Test getting notification for coin that is about to be created
-        await time_out_assert(10, wallet_is_synced, True, wallet_node, full_node_api)
+        await time_out_assert(20, wallet_is_synced, True, wallet_node, full_node_api)
         tx_record = await standard_wallet.generate_signed_transaction(uint64(10), puzzle_hash, uint64(0))
 
         tx_record.spend_bundle.additions()
@@ -348,7 +348,7 @@ class TestSimpleSyncProtocol:
     async def test_subscribe_for_ph_reorg(self, wallet_node_simulator, self_hostname):
         num_blocks = 4
         long_blocks = 20
-        full_nodes, wallets = wallet_node_simulator
+        full_nodes, wallets, _ = wallet_node_simulator
         full_node_api = full_nodes[0]
         wallet_node, server_2 = wallets[0]
         fn_server = full_node_api.full_node.server
@@ -378,12 +378,12 @@ class TestSimpleSyncProtocol:
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(zero_ph))
 
         expected_height = uint32(long_blocks + 2 * num_blocks + 1)
-        await time_out_assert(15, full_node_api.full_node.blockchain.get_peak_height, expected_height)
+        await time_out_assert(20, full_node_api.full_node.blockchain.get_peak_height, expected_height)
 
         coin_records = await full_node_api.full_node.coin_store.get_coin_records_by_puzzle_hash(True, puzzle_hash)
         assert len(coin_records) > 0
         fork_height = expected_height - num_blocks - 5
-        req = ReorgProtocol(fork_height, expected_height + 5, zero_ph)
+        req = ReorgProtocol(fork_height, expected_height + 5, zero_ph, None)
         await full_node_api.reorg_from_index_to_new_index(req)
 
         coin_records = await full_node_api.full_node.coin_store.get_coin_records_by_puzzle_hash(True, puzzle_hash)
@@ -423,7 +423,7 @@ class TestSimpleSyncProtocol:
     async def test_subscribe_for_coin_id_reorg(self, wallet_node_simulator, self_hostname):
         num_blocks = 4
         long_blocks = 20
-        full_nodes, wallets = wallet_node_simulator
+        full_nodes, wallets, _ = wallet_node_simulator
         full_node_api = full_nodes[0]
         wallet_node, server_2 = wallets[0]
         fn_server = full_node_api.full_node.server
@@ -450,7 +450,7 @@ class TestSimpleSyncProtocol:
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(zero_ph))
 
         expected_height = uint32(long_blocks + 2 * num_blocks + 1)
-        await time_out_assert(15, full_node_api.full_node.blockchain.get_peak_height, expected_height)
+        await time_out_assert(20, full_node_api.full_node.blockchain.get_peak_height, expected_height)
 
         coin_records = await full_node_api.full_node.coin_store.get_coin_records_by_puzzle_hash(True, puzzle_hash)
         assert len(coin_records) > 0
@@ -461,7 +461,7 @@ class TestSimpleSyncProtocol:
             assert msg_response is not None
 
         fork_height = expected_height - num_blocks - 5
-        req = ReorgProtocol(fork_height, expected_height + 5, zero_ph)
+        req = ReorgProtocol(fork_height, expected_height + 5, zero_ph, None)
         await full_node_api.reorg_from_index_to_new_index(req)
 
         coin_records = await full_node_api.full_node.coin_store.get_coin_records_by_puzzle_hash(True, puzzle_hash)
@@ -487,9 +487,9 @@ class TestSimpleSyncProtocol:
         assert second_coin.created_height is None
 
     @pytest.mark.asyncio
-    async def test_subscribe_for_hint(self, bt, wallet_node_simulator, self_hostname):
+    async def test_subscribe_for_hint(self, wallet_node_simulator, self_hostname):
         num_blocks = 4
-        full_nodes, wallets = wallet_node_simulator
+        full_nodes, wallets, bt = wallet_node_simulator
         full_node_api = full_nodes[0]
         wallet_node, server_2 = wallets[0]
         fn_server = full_node_api.full_node.server
@@ -523,7 +523,7 @@ class TestSimpleSyncProtocol:
                 ConditionWithArgs(ConditionOpcode.CREATE_COIN, [hint_puzzle_hash, amount_bin, hint])
             ]
         }
-        await time_out_assert(10, wallet_is_synced, True, wallet_node, full_node_api)
+        await time_out_assert(20, wallet_is_synced, True, wallet_node, full_node_api)
         tx: SpendBundle = wt.generate_signed_transaction(
             10,
             wt.get_new_puzzlehash(),
@@ -532,7 +532,7 @@ class TestSimpleSyncProtocol:
         )
         await full_node_api.respond_transaction(RespondTransaction(tx), fake_wallet_peer)
 
-        await time_out_assert(15, tx_in_pool, True, full_node_api.full_node.mempool_manager, tx.name())
+        await time_out_assert(20, tx_in_pool, True, full_node_api.full_node.mempool_manager, tx.name())
 
         for i in range(0, num_blocks):
             await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph))
@@ -562,9 +562,9 @@ class TestSimpleSyncProtocol:
         assert data_response.coin_states[0] == coin_records[0].coin_state
 
     @pytest.mark.asyncio
-    async def test_subscribe_for_hint_long_sync(self, wallet_two_node_simulator, bt, self_hostname):
+    async def test_subscribe_for_hint_long_sync(self, wallet_two_node_simulator, self_hostname):
         num_blocks = 4
-        full_nodes, wallets = wallet_two_node_simulator
+        full_nodes, wallets, bt = wallet_two_node_simulator
         full_node_api = full_nodes[0]
         full_node_api_1 = full_nodes[1]
 
@@ -606,7 +606,7 @@ class TestSimpleSyncProtocol:
                 ConditionWithArgs(ConditionOpcode.CREATE_COIN, [hint_puzzle_hash, amount_bin, hint])
             ]
         }
-        await time_out_assert(10, wallet_is_synced, True, wallet_node, full_node_api)
+        await time_out_assert(20, wallet_is_synced, True, wallet_node, full_node_api)
         tx: SpendBundle = wt.generate_signed_transaction(
             10,
             wt.get_new_puzzlehash(),
@@ -615,7 +615,7 @@ class TestSimpleSyncProtocol:
         )
         await full_node_api.respond_transaction(RespondTransaction(tx), fake_wallet_peer)
 
-        await time_out_assert(15, tx_in_pool, True, full_node_api.full_node.mempool_manager, tx.name())
+        await time_out_assert(20, tx_in_pool, True, full_node_api.full_node.mempool_manager, tx.name())
 
         # Create more blocks than recent "short_sync_blocks_behind_threshold" so that node enters batch
         for i in range(0, 100):
@@ -626,7 +626,7 @@ class TestSimpleSyncProtocol:
 
         await fn_server_1.start_client(PeerInfo(self_hostname, uint16(fn_server._port)), None)
         node0_height = full_node_api.full_node.blockchain.get_peak_height()
-        await time_out_assert(15, full_node_api_1.full_node.blockchain.get_peak_height, node0_height)
+        await time_out_assert(60, full_node_api_1.full_node.blockchain.get_peak_height, node0_height)
 
         all_messages = await get_all_messages_in_queue(incoming_queue)
         all_messages_1 = await get_all_messages_in_queue(incoming_queue_1)
