@@ -1,11 +1,15 @@
+from __future__ import annotations
+
 from typing import List, Optional, Set
 
 import pytest
+from aiosqlite import Error as AIOSqliteError
 from colorlog import getLogger
 
 from chinilla.consensus.block_record import BlockRecord
 from chinilla.consensus.block_rewards import calculate_base_farmer_reward, calculate_pool_reward
 from chinilla.full_node.full_node_api import FullNodeAPI
+from chinilla.full_node.mempool_manager import MempoolManager
 from chinilla.full_node.weight_proof import WeightProofHandler
 from chinilla.protocols import full_node_protocol, wallet_protocol
 from chinilla.protocols.protocol_message_types import ProtocolMessageTypes
@@ -15,6 +19,7 @@ from chinilla.server.outbound_message import Message
 from chinilla.simulator.simulator_protocol import FarmNewBlockProtocol
 from chinilla.simulator.time_out_assert import time_out_assert, time_out_assert_not_none
 from chinilla.types.blockchain_format.program import Program
+from chinilla.types.blockchain_format.sized_bytes import bytes32
 from chinilla.types.peer_info import PeerInfo
 from chinilla.util.block_cache import BlockCache
 from chinilla.util.hash import std_hash
@@ -22,6 +27,7 @@ from chinilla.util.ints import uint16, uint32, uint64
 from chinilla.wallet.nft_wallet.nft_wallet import NFTWallet
 from chinilla.wallet.transaction_record import TransactionRecord
 from chinilla.wallet.util.compute_memos import compute_memos
+from chinilla.wallet.util.wallet_sync_utils import PeerRequestException
 from chinilla.wallet.util.wallet_types import AmountWithPuzzlehash
 from chinilla.wallet.wallet_coin_record import WalletCoinRecord
 from chinilla.wallet.wallet_weight_proof_handler import get_wp_fork_point
@@ -39,7 +45,7 @@ async def wallet_height_at_least(wallet_node, h):
 
 
 async def get_nft_count(wallet: NFTWallet) -> int:
-    return len(await wallet.load_current_nft())
+    return await wallet.get_nft_count()
 
 
 log = getLogger(__name__)
@@ -370,8 +376,8 @@ class TestWalletSync:
 
         for wallet_node, wallet_server in wallets:
             wallet = wallet_node.wallet_state_manager.main_wallet
-            await time_out_assert(20, wallet.get_confirmed_balance, funds)
-            await time_out_assert(20, get_tx_count, 2 * (num_blocks - 1), wallet_node.wallet_state_manager, 1)
+            await time_out_assert(60, wallet.get_confirmed_balance, funds)
+            await time_out_assert(60, get_tx_count, 2 * (num_blocks - 1), wallet_node.wallet_state_manager, 1)
 
         # Reorg blocks that carry reward
         num_blocks = 30
@@ -382,8 +388,8 @@ class TestWalletSync:
 
         for wallet_node, wallet_server in wallets:
             wallet = wallet_node.wallet_state_manager.main_wallet
-            await time_out_assert(20, get_tx_count, 0, wallet_node.wallet_state_manager, 1)
-            await time_out_assert(20, wallet.get_confirmed_balance, 0)
+            await time_out_assert(60, get_tx_count, 0, wallet_node.wallet_state_manager, 1)
+            await time_out_assert(60, wallet.get_confirmed_balance, 0)
 
     @pytest.mark.asyncio
     async def test_wallet_reorg_get_coinbase(self, two_wallet_nodes, default_400_blocks, self_hostname):
@@ -576,6 +582,7 @@ class TestWalletSync:
 
     @pytest.mark.asyncio
     async def test_get_wp_fork_point(self, default_10000_blocks):
+
         blocks = default_10000_blocks
         header_cache, height_to_hash, sub_blocks, summaries = await load_blocks_dont_validate(blocks)
         wpf = WeightProofHandler(test_constants, BlockCache(sub_blocks, header_cache, height_to_hash, summaries))
@@ -584,6 +591,7 @@ class TestWalletSync:
         wp3 = await wpf.get_proof_of_weight(header_cache[height_to_hash[uint32(7500)]].header_hash)
         wp4 = await wpf.get_proof_of_weight(header_cache[height_to_hash[uint32(8700)]].header_hash)
         wp5 = await wpf.get_proof_of_weight(header_cache[height_to_hash[uint32(9700)]].header_hash)
+        wp6 = await wpf.get_proof_of_weight(header_cache[height_to_hash[uint32(9010)]].header_hash)
         fork12 = get_wp_fork_point(test_constants, wp1, wp2)
         fork13 = get_wp_fork_point(test_constants, wp3, wp1)
         fork14 = get_wp_fork_point(test_constants, wp4, wp1)
@@ -591,13 +599,38 @@ class TestWalletSync:
         fork24 = get_wp_fork_point(test_constants, wp4, wp2)
         fork34 = get_wp_fork_point(test_constants, wp3, wp4)
         fork45 = get_wp_fork_point(test_constants, wp4, wp5)
-        assert fork14 == 8700
-        assert fork24 == 8700
-        assert fork12 == 9000
+        fork16 = get_wp_fork_point(test_constants, wp1, wp6)
+
+        # overlap between recent chain in wps, fork point is the tip of the shorter wp
+        assert fork12 == wp1.recent_chain_data[-1].height
+        assert fork16 == wp1.recent_chain_data[-1].height
+
+        # if there is an overlap between the recent chains we can find the exact fork point
+        # if not we should get the latest block with a sub epoch summary that exists in both wp's
+        # this can happen in fork24 and fork14 since they are not very far and also not very close
+
+        if wp2.recent_chain_data[0].height > wp4.recent_chain_data[-1].height:
+            assert fork24 in summaries.keys()
+            assert fork24 < wp4.recent_chain_data[-1].height
+        else:
+            assert fork24 == wp4.recent_chain_data[-1].height
+
+        if wp1.recent_chain_data[0].height > wp4.recent_chain_data[-1].height:
+            assert fork14 in summaries.keys()
+            assert fork14 < wp4.recent_chain_data[-1].height
+        else:
+            assert fork14 == wp4.recent_chain_data[-1].height
+
+        # no overlap between recent chain in wps, fork point
+        # is the latest block with a sub epoch summary that exists in both wp's
         assert fork13 in summaries.keys()
+        assert fork13 < wp3.recent_chain_data[-1].height
         assert fork23 in summaries.keys()
+        assert fork23 < wp3.recent_chain_data[-1].height
         assert fork34 in summaries.keys()
+        assert fork23 < wp3.recent_chain_data[-1].height
         assert fork45 in summaries.keys()
+        assert fork45 < wp4.recent_chain_data[-1].height
 
     """
     This tests that a wallet filters out the dust properly.
@@ -1081,7 +1114,7 @@ class TestWalletSync:
         # Create a new NFT and send it to the farmer's NFT wallet
         metadata = Program.to(
             [
-                ("u", ["https://www.chinilla.net/img/branding/chinilla-logo.svg"]),
+                ("u", ["https://www.chinilla.com/img/branding/chinilla-logo.svg"]),
                 ("h", "0xD4584AD463139FA8C0D9F68F4B59F185"),
             ]
         )
@@ -1108,11 +1141,12 @@ class TestWalletSync:
         await time_out_assert(15, get_nft_count, 1, farm_nft_wallet)
         await time_out_assert(15, get_nft_count, 0, dust_nft_wallet)
 
+        nft_coins = await farm_nft_wallet.get_current_nfts()
         # Send the NFT to the dust wallet
         txs = await farm_nft_wallet.generate_signed_transaction(
-            [uint64(farm_nft_wallet.my_nft_coins[0].coin.amount)],
+            [uint64(nft_coins[0].coin.amount)],
             [dust_ph],
-            coins={farm_nft_wallet.my_nft_coins[0].coin},
+            coins={nft_coins[0].coin},
         )
         assert len(txs) == 1
         assert txs[0].spend_bundle is not None
@@ -1136,3 +1170,130 @@ class TestWalletSync:
         # The dust wallet should now hold the NFT. It should not be filtered
         await time_out_assert(15, get_nft_count, 0, farm_nft_wallet)
         await time_out_assert(15, get_nft_count, 1, dust_nft_wallet)
+
+    @pytest.mark.asyncio
+    async def test_retry_store(self, two_wallet_nodes, self_hostname):
+        full_nodes, wallets, bt = two_wallet_nodes
+        full_node_api = full_nodes[0]
+        full_node_server = full_node_api.full_node.server
+
+        await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(bytes32([0] * 32)))
+
+        # Trusted node sync
+        wallets[0][0].config["trusted_peers"] = {full_node_server.node_id.hex(): full_node_server.node_id.hex()}
+
+        # Untrusted node sync
+        wallets[1][0].config["trusted_peers"] = {}
+
+        def flaky_get_coin_state(node, func):
+            async def new_func(*args, **kwargs):
+                if node.coin_state_flaky:
+                    node.coin_state_flaky = False
+                    raise PeerRequestException()
+                else:
+                    return await func(*args, **kwargs)
+
+            return new_func
+
+        def flaky_fetch_puzzle_solution(node, func):
+            async def new_func(*args, **kwargs):
+                if node.puzzle_solution_flaky:
+                    node.puzzle_solution_flaky = False
+                    raise PeerRequestException()
+                else:
+                    return await func(*args, **kwargs)
+
+            return new_func
+
+        def flaky_fetch_children(node, func):
+            async def new_func(*args, **kwargs):
+                if node.fetch_children_flaky:
+                    node.fetch_children_flaky = False
+                    raise PeerRequestException()
+                else:
+                    return await func(*args, **kwargs)
+
+            return new_func
+
+        def flaky_get_timestamp(node, func):
+            async def new_func(*args, **kwargs):
+                if node.get_timestamp_flaky:
+                    node.get_timestamp_flaky = False
+                    raise PeerRequestException()
+                else:
+                    return await func(*args, **kwargs)
+
+            return new_func
+
+        def flaky_info_for_puzhash(node, func):
+            async def new_func(*args, **kwargs):
+                if node.db_flaky:
+                    node.db_flaky = False
+                    raise AIOSqliteError()
+                else:
+                    return await func(*args, **kwargs)
+
+            return new_func
+
+        for wallet_node, wallet_server in wallets:
+            wallet_node.coin_state_flaky = True
+            wallet_node.puzzle_solution_flaky = True
+            wallet_node.fetch_children_flaky = True
+            wallet_node.get_timestamp_flaky = True
+            wallet_node.db_flaky = True
+
+            wallet_node.get_coin_state = flaky_get_coin_state(wallet_node, wallet_node.get_coin_state)
+            wallet_node.fetch_puzzle_solution = flaky_fetch_puzzle_solution(
+                wallet_node, wallet_node.fetch_puzzle_solution
+            )
+            wallet_node.fetch_children = flaky_fetch_children(wallet_node, wallet_node.fetch_children)
+            wallet_node.get_timestamp_for_height = flaky_get_timestamp(
+                wallet_node, wallet_node.get_timestamp_for_height
+            )
+            wallet_node.wallet_state_manager.puzzle_store.wallet_info_for_puzzle_hash = flaky_info_for_puzhash(
+                wallet_node, wallet_node.wallet_state_manager.puzzle_store.wallet_info_for_puzzle_hash
+            )
+
+            await wallet_server.start_client(PeerInfo(self_hostname, uint16(full_node_server._port)), None)
+
+            wallet = wallet_node.wallet_state_manager.main_wallet
+            ph = await wallet.get_new_puzzlehash()
+            await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(ph))
+            await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(bytes32([0] * 32)))
+
+            async def len_gt_0(func, *args):
+                return len((await func(*args))) > 0
+
+            await time_out_assert(
+                15, len_gt_0, True, wallet_node.wallet_state_manager.retry_store.get_all_states_to_retry
+            )
+            await time_out_assert(
+                30, len_gt_0, False, wallet_node.wallet_state_manager.retry_store.get_all_states_to_retry
+            )
+
+            await time_out_assert(30, wallet.get_confirmed_balance, 2_000_000_000_000)
+
+            tx = await wallet.generate_signed_transaction(1_000_000_000_000, bytes32([0] * 32), memos=[ph])
+            await wallet_node.wallet_state_manager.add_pending_transaction(tx)
+
+            async def tx_in_pool(mempool: MempoolManager, tx_id: bytes32):
+                tx = mempool.get_spendbundle(tx_id)
+                if tx is None:
+                    return False
+                return True
+
+            await time_out_assert(15, tx_in_pool, True, full_node_api.full_node.mempool_manager, tx.name)
+            await full_node_api.farm_new_transaction_block(FarmNewBlockProtocol(bytes32([0] * 32)))
+
+            await time_out_assert(
+                15, len_gt_0, True, wallet_node.wallet_state_manager.retry_store.get_all_states_to_retry
+            )
+            await time_out_assert(
+                120, len_gt_0, False, wallet_node.wallet_state_manager.retry_store.get_all_states_to_retry
+            )
+            assert not wallet_node.coin_state_flaky
+            assert not wallet_node.puzzle_solution_flaky
+            assert not wallet_node.fetch_children_flaky
+            assert not wallet_node.get_timestamp_flaky
+            assert not wallet_node.db_flaky
+            await time_out_assert(30, wallet.get_confirmed_balance, 1_000_000_000_000)
