@@ -1,7 +1,7 @@
 from dataclasses import dataclass
 from typing import Any, Callable, Dict, List, Optional, Tuple
 
-from blspy import G1Element, G2Element
+from chinillablspy import G1Element, G2Element
 from clvm_tools import binutils
 
 from chinilla.types.blockchain_format.program import Program, SerializedProgram
@@ -766,5 +766,3 @@ def test_lifecycle_with_coinstore_as_wallet():
     assert update_count == 0
 
     assert len(list(coin_store.all_unspent_coins())) == 2
-
-    return 0
